# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/galacteek.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_GalacteekWindow(object):
    def setupUi(self, GalacteekWindow):
        GalacteekWindow.setObjectName("GalacteekWindow")
        GalacteekWindow.resize(697, 446)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(GalacteekWindow.sizePolicy().hasHeightForWidth())
        GalacteekWindow.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/share/icons/ipfs-logo-128-ice-text.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        GalacteekWindow.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(GalacteekWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.openBrowserTabButton = QtWidgets.QPushButton(self.centralwidget)
        self.openBrowserTabButton.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/share/icons/ipfs-logo-128-ice.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.openBrowserTabButton.setIcon(icon1)
        self.openBrowserTabButton.setIconSize(QtCore.QSize(24, 24))
        self.openBrowserTabButton.setObjectName("openBrowserTabButton")
        self.horizontalLayout_2.addWidget(self.openBrowserTabButton)
        self.myFilesButton = QtWidgets.QToolButton(self.centralwidget)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/share/icons/folder-open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.myFilesButton.setIcon(icon2)
        self.myFilesButton.setIconSize(QtCore.QSize(24, 24))
        self.myFilesButton.setObjectName("myFilesButton")
        self.horizontalLayout_2.addWidget(self.myFilesButton)
        self.mediaPlayerButton = QtWidgets.QToolButton(self.centralwidget)
        self.mediaPlayerButton.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/share/icons/multimedia.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.mediaPlayerButton.setIcon(icon3)
        self.mediaPlayerButton.setIconSize(QtCore.QSize(24, 24))
        self.mediaPlayerButton.setObjectName("mediaPlayerButton")
        self.horizontalLayout_2.addWidget(self.mediaPlayerButton)
        spacerItem = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.manageKeysButton = QtWidgets.QToolButton(self.centralwidget)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/share/icons/lock-and-key.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.manageKeysButton.setIcon(icon4)
        self.manageKeysButton.setIconSize(QtCore.QSize(24, 24))
        self.manageKeysButton.setObjectName("manageKeysButton")
        self.horizontalLayout_2.addWidget(self.manageKeysButton, 0, QtCore.Qt.AlignLeft)
        self.hashmarksButton = QtWidgets.QPushButton(self.centralwidget)
        self.hashmarksButton.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/share/icons/hashmarks.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.hashmarksButton.setIcon(icon5)
        self.hashmarksButton.setIconSize(QtCore.QSize(24, 24))
        self.hashmarksButton.setObjectName("hashmarksButton")
        self.horizontalLayout_2.addWidget(self.hashmarksButton)
        spacerItem1 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.writeNewDocumentButton = QtWidgets.QPushButton(self.centralwidget)
        self.writeNewDocumentButton.setText("")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/share/icons/write-document.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.writeNewDocumentButton.setIcon(icon6)
        self.writeNewDocumentButton.setIconSize(QtCore.QSize(24, 24))
        self.writeNewDocumentButton.setObjectName("writeNewDocumentButton")
        self.horizontalLayout_2.addWidget(self.writeNewDocumentButton)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setSpacing(4)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.hLayoutProfile = QtWidgets.QHBoxLayout()
        self.hLayoutProfile.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.hLayoutProfile.setSpacing(4)
        self.hLayoutProfile.setObjectName("hLayoutProfile")
        self.peersMgrButton = QtWidgets.QToolButton(self.centralwidget)
        self.peersMgrButton.setText("")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/share/icons/peers.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.peersMgrButton.setIcon(icon7)
        self.peersMgrButton.setIconSize(QtCore.QSize(24, 24))
        self.peersMgrButton.setObjectName("peersMgrButton")
        self.hLayoutProfile.addWidget(self.peersMgrButton)
        self.profileEditButton = QtWidgets.QToolButton(self.centralwidget)
        self.profileEditButton.setText("")
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/share/icons/profile-user.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.profileEditButton.setIcon(icon8)
        self.profileEditButton.setIconSize(QtCore.QSize(24, 24))
        self.profileEditButton.setObjectName("profileEditButton")
        self.hLayoutProfile.addWidget(self.profileEditButton)
        self.horizontalLayout_4.addLayout(self.hLayoutProfile)
        self.label = QtWidgets.QLabel(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setTextFormat(QtCore.Qt.PlainText)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setWordWrap(False)
        self.label.setIndent(0)
        self.label.setTextInteractionFlags(QtCore.Qt.NoTextInteraction)
        self.label.setObjectName("label")
        self.horizontalLayout_4.addWidget(self.label)
        self.ipfsSearch = QtWidgets.QLineEdit(self.centralwidget)
        self.ipfsSearch.setMaximumSize(QtCore.QSize(230, 16777215))
        self.ipfsSearch.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.ipfsSearch.setMaxLength(128)
        self.ipfsSearch.setObjectName("ipfsSearch")
        self.horizontalLayout_4.addWidget(self.ipfsSearch)
        self.ipfsSearchButton = QtWidgets.QToolButton(self.centralwidget)
        self.ipfsSearchButton.setText("")
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/share/icons/ipfs-search.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.ipfsSearchButton.setIcon(icon9)
        self.ipfsSearchButton.setIconSize(QtCore.QSize(24, 24))
        self.ipfsSearchButton.setObjectName("ipfsSearchButton")
        self.horizontalLayout_4.addWidget(self.ipfsSearchButton)
        self.pinAllGlobalButton = QtWidgets.QToolButton(self.centralwidget)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/share/icons/pin-black.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pinAllGlobalButton.setIcon(icon10)
        self.pinAllGlobalButton.setIconSize(QtCore.QSize(24, 24))
        self.pinAllGlobalButton.setObjectName("pinAllGlobalButton")
        self.horizontalLayout_4.addWidget(self.pinAllGlobalButton)
        self.clipboardMultiLoader = QtWidgets.QToolButton(self.centralwidget)
        self.clipboardMultiLoader.setText("")
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/share/icons/clipboard.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.clipboardMultiLoader.setIcon(icon11)
        self.clipboardMultiLoader.setIconSize(QtCore.QSize(24, 24))
        self.clipboardMultiLoader.setObjectName("clipboardMultiLoader")
        self.horizontalLayout_4.addWidget(self.clipboardMultiLoader)
        self.horizontalLayout_2.addLayout(self.horizontalLayout_4)
        self.horizontalLayout.addLayout(self.horizontalLayout_2)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.tabBlank = QtWidgets.QWidget()
        self.tabBlank.setObjectName("tabBlank")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.tabBlank)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.tabWidget.addTab(self.tabBlank, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.verticalLayout_3.addLayout(self.verticalLayout)
        GalacteekWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(GalacteekWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 697, 20))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuEdit = QtWidgets.QMenu(self.menubar)
        self.menuEdit.setObjectName("menuEdit")
        self.menuUser_Profile = QtWidgets.QMenu(self.menuEdit)
        self.menuUser_Profile.setObjectName("menuUser_Profile")
        self.menuAbout = QtWidgets.QMenu(self.menubar)
        self.menuAbout.setObjectName("menuAbout")
        self.menuTools = QtWidgets.QMenu(self.menubar)
        self.menuTools.setObjectName("menuTools")
        GalacteekWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(GalacteekWindow)
        self.statusbar.setObjectName("statusbar")
        GalacteekWindow.setStatusBar(self.statusbar)
        self.actionQuit = QtWidgets.QAction(GalacteekWindow)
        self.actionQuit.setPriority(QtWidgets.QAction.LowPriority)
        self.actionQuit.setObjectName("actionQuit")
        self.actionAddProfile = QtWidgets.QAction(GalacteekWindow)
        self.actionAddProfile.setObjectName("actionAddProfile")
        self.actionShowPeersInformation = QtWidgets.QAction(GalacteekWindow)
        self.actionShowPeersInformation.setObjectName("actionShowPeersInformation")
        self.actionShowIpfsRefs = QtWidgets.QAction(GalacteekWindow)
        self.actionShowIpfsRefs.setObjectName("actionShowIpfsRefs")
        self.actionSettings = QtWidgets.QAction(GalacteekWindow)
        self.actionSettings.setObjectName("actionSettings")
        self.actionAboutGalacteek = QtWidgets.QAction(GalacteekWindow)
        self.actionAboutGalacteek.setObjectName("actionAboutGalacteek")
        self.actionCloseAllTabs = QtWidgets.QAction(GalacteekWindow)
        self.actionCloseAllTabs.setPriority(QtWidgets.QAction.HighPriority)
        self.actionCloseAllTabs.setObjectName("actionCloseAllTabs")
        self.actionNew_Profile = QtWidgets.QAction(GalacteekWindow)
        self.actionNew_Profile.setObjectName("actionNew_Profile")
        self.actionDonate = QtWidgets.QAction(GalacteekWindow)
        self.actionDonate.setObjectName("actionDonate")
        self.actionEvent_log = QtWidgets.QAction(GalacteekWindow)
        self.actionEvent_log.setObjectName("actionEvent_log")
        self.menuFile.addAction(self.actionQuit)
        self.menuFile.addAction(self.actionCloseAllTabs)
        self.menuUser_Profile.addAction(self.actionNew_Profile)
        self.menuEdit.addAction(self.actionSettings)
        self.menuEdit.addAction(self.menuUser_Profile.menuAction())
        self.menuAbout.addAction(self.actionAboutGalacteek)
        self.menuAbout.addAction(self.actionDonate)
        self.menuTools.addAction(self.actionEvent_log)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuEdit.menuAction())
        self.menubar.addAction(self.menuTools.menuAction())
        self.menubar.addAction(self.menuAbout.menuAction())

        self.retranslateUi(GalacteekWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(GalacteekWindow)

    def retranslateUi(self, GalacteekWindow):
        _translate = QtCore.QCoreApplication.translate
        GalacteekWindow.setWindowTitle(_translate("GalacteekWindow", "Galacteek"))
        self.openBrowserTabButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Browse</p></body></html>"))
        self.myFilesButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>File Manager</p></body></html>"))
        self.myFilesButton.setText(_translate("GalacteekWindow", "..."))
        self.mediaPlayerButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Media Player</p></body></html>"))
        self.manageKeysButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>IPFS keys</p></body></html>"))
        self.manageKeysButton.setText(_translate("GalacteekWindow", "..."))
        self.hashmarksButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Hashmarks</p></body></html>"))
        self.writeNewDocumentButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Write and import text document</p></body></html>"))
        self.peersMgrButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Peers</p></body></html>"))
        self.profileEditButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>User Profile</p></body></html>"))
        self.label.setText(_translate("GalacteekWindow", "Search"))
        self.ipfsSearchButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Search content on IPFS (ipfs-search.com)</p></body></html>"))
        self.pinAllGlobalButton.setToolTip(_translate("GalacteekWindow", "<html><head/><body><p>Global automatic pinning (enables automatic pinning for all browsing sessions)</p></body></html>"))
        self.pinAllGlobalButton.setText(_translate("GalacteekWindow", "..."))
        self.menuFile.setTitle(_translate("GalacteekWindow", "File"))
        self.menuEdit.setTitle(_translate("GalacteekWindow", "Edit"))
        self.menuUser_Profile.setTitle(_translate("GalacteekWindow", "User Profile"))
        self.menuAbout.setTitle(_translate("GalacteekWindow", "Help"))
        self.menuTools.setTitle(_translate("GalacteekWindow", "Tools"))
        self.actionQuit.setText(_translate("GalacteekWindow", "Quit"))
        self.actionQuit.setShortcut(_translate("GalacteekWindow", "Ctrl+Q"))
        self.actionAddProfile.setText(_translate("GalacteekWindow", "Add Profile"))
        self.actionShowPeersInformation.setText(_translate("GalacteekWindow", "Show peers information"))
        self.actionShowIpfsRefs.setText(_translate("GalacteekWindow", "Show refs"))
        self.actionSettings.setText(_translate("GalacteekWindow", "Settings"))
        self.actionAboutGalacteek.setText(_translate("GalacteekWindow", "About"))
        self.actionCloseAllTabs.setText(_translate("GalacteekWindow", "Close all tabs"))
        self.actionNew_Profile.setText(_translate("GalacteekWindow", "New Profile"))
        self.actionDonate.setText(_translate("GalacteekWindow", "Donate"))
        self.actionEvent_log.setText(_translate("GalacteekWindow", "Event log"))

from . import galacteek_rc




from PyQt5.QtCore import (Qt, QEvent, QObject, pyqtSignal, QFile)
from PyQt5.QtWidgets import QWidget, QApplication

from galacteek.ipfs.wrappers import ipfsOp

class GalacteekTab(QWidget):
    def __init__(self, gWindow, parent=None, **kw):
        super().__init__(parent=parent)

        self.gWindow = gWindow

    def onClose(self):
        return True

    @ipfsOp
    async def initialize(self, op):
        pass

    @property
    def app(self):
        return self.gWindow.app

    @property
    def loop(self):
        return self.app.loop

    @property
    def profile(self):
        return self.app.ipfsCtx.currentProfile




class P2PService:
    def __init__(self, name, description, protocolName):
        self._name = name
        self._description = description
        self._protocolName = protocolName
        self._listener = None

    @property
    def name(self):
        return self._name

    @property
    def description(self):
        return self._description

    @property
    def protocolName(self):
        return self._protocolName

    @property
    def listener(self):
        return self._listener

    async def createListener(self, *args, **kw):
        pass

    async def stop(self):
        if self.listener:
            await self.listener.close()




import sys
import time
import os.path
import asyncio
import mimetypes

from PyQt5.QtWidgets import (QWidget, QFrame, QApplication, QMainWindow,
        QDialog, QLabel, QTextEdit, QPushButton, QVBoxLayout, QAction,
        QTabWidget, QFileDialog, QTreeWidget, QTreeWidgetItem,
        QTreeView, QTreeWidgetItem, QMessageBox, QMenu, QAbstractItemView,
        QShortcut, QInputDialog, QToolButton, QHeaderView, QFileSystemModel)

from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtGui import QPixmap, QIcon, QClipboard, QKeySequence

from PyQt5.QtCore import (QCoreApplication, QUrl, Qt, QEvent, QObject,
    pyqtSignal,QBuffer, QModelIndex, QMimeData, QFile, QStandardPaths,
    QDir)
from PyQt5.Qt import QByteArray

from quamash import QEventLoop, QThreadExecutor

from galacteek.ipfs.ipfsops import *
from galacteek.ipfs.wrappers import ipfsOp
from galacteek.ipfs.cache import IPFSEntryCache
from galacteek.appsettings import *

from . import ui_files
from . import mediaplayer
from . import ipfsview
from . import galacteek_rc
from . import modelhelpers
from .i18n import *
from .helpers import *
from .widgets import GalacteekTab
from .hashmarks import *

import aioipfs

# Files messages
def iFileImportError():
    return QCoreApplication.translate('FileManagerForm', 'Error importing file {}')

def iCopyHashToSelClipboard():
    return QCoreApplication.translate('FileManagerForm',
        "Copy file's hash to selection clipboard")

def iCopyHashToGlobalClipboard():
    return QCoreApplication.translate('FileManagerForm',
        "Copy file's hash to global clipboard")

def iAddedFile(name):
    return QCoreApplication.translate('FileManagerForm',
            'Added file {0}').format(name)
def iLoadingFile(name):
    return QCoreApplication.translate('FileManagerForm',
            'Loading file {0}').format(name)
def iLoading(name):
    return QCoreApplication.translate('FileManagerForm',
            'Loading {0}').format(name)

def iOpenWith():
    return QCoreApplication.translate('FileManagerForm', 'Open with')

def iDeleteFile():
    return QCoreApplication.translate('FileManagerForm', 'Delete file')

def iExploreDir():
    return QCoreApplication.translate('FileManagerForm', 'Explore directory')

def iUnlinkFile():
    return QCoreApplication.translate('FileManagerForm', 'Unlink file')

def iHashmarkFile():
    return QCoreApplication.translate('FileManagerForm', 'Hashmark')

def iBrowseFile():
    return QCoreApplication.translate('FileManagerForm', 'Browse')

def iSelectDirectory():
    return QCoreApplication.translate('FileManagerForm', 'Select directory')

def iSelectFiles():
    return QCoreApplication.translate('FileManagerForm',
        'Select one or more files to import')

def iMusic():
    return QCoreApplication.translate('FileManagerForm', 'Music')

def iPictures():
    return QCoreApplication.translate('FileManagerForm', 'Pictures')

def iVideos():
    return QCoreApplication.translate('FileManagerForm', 'Videos')

def iHome():
    return QCoreApplication.translate('FileManagerForm', 'Home')

def iCode():
    return QCoreApplication.translate('FileManagerForm', 'Code')

def iDocuments():
    return QCoreApplication.translate('FileManagerForm', 'Documents')

class IPFSItem(UneditableItem):
    def __init__(self, text, path=None, parenthash=None, icon=None):
        super(IPFSItem, self).__init__(text, icon=icon)
        self.path = path
        self.setParentHash(parenthash)

    def setParentHash(self, pHash):
        self.parentHash = pHash

    def getParentHash(self):
        return self.parentHash

    def setPath(self, path):
        self.path = path

    def getPath(self):
        return self.path

class IPFSNameItem(IPFSItem):
    def __init__(self, entry, text, icon):
        super().__init__(text, icon=icon)

        self._entry = entry
        self._mimeType = None

    @property
    def entry(self):
        return self._entry

    @property
    def mimeType(self):
        return self._mimeType

    def mimeFromDb(self, db):
        self._mimeType = db.mimeTypeForFile(self.entry['Name'])

    @property
    def mimeCategory(self):
        if self.mimeType:
            return self.mimeTypeName.split('/')[0]

    @property
    def mimeTypeName(self):
        if self.mimeType:
            return self.mimeType.name()

    def isFile(self):
        return self.entry['Type'] == 0

    def isDir(self):
        return self.entry['Type'] == 1

class FilesItemModel(QStandardItemModel):
    fileDropEvent = pyqtSignal(str)
    directoryDropEvent = pyqtSignal(str)

    def __init__(self):
        QStandardItemModel.__init__(self)

        self.itemRoot = self.invisibleRootItem()
        self.itemRootIdx = self.indexFromItem(self.itemRoot)
        self.entryCache = IPFSEntryCache()

        self.rowsInserted.connect(self.onRowsInserted)
        self.rowsAboutToBeRemoved.connect(self.onRowsToBeRemoved)

    def setupItemsFromProfile(self, profile):
        self.itemHome = IPFSItem(iHome(), path=profile.pathHome)
        self.itemPictures = IPFSItem(iPictures(),
                path=profile.pathPictures)
        self.itemVideos = IPFSItem(iVideos(),
                path=profile.pathVideos)
        self.itemMusic = IPFSItem(iMusic(),
                path=profile.pathMusic)
        self.itemCode = IPFSItem(iCode(),
                path=profile.pathCode)
        self.itemDocuments = IPFSItem(iDocuments(),
                path=profile.pathDocuments)

        self.itemRoot.appendRows([
            self.itemHome,
            self.itemPictures,
            self.itemVideos,
            self.itemCode,
            self.itemMusic,
            self.itemDocuments
        ])

    def displayItem(self, arg):
        self.itemRoot.appendRow(arg)

    def onRowsInserted(self, parent, first, last):
        for itNum in range(first, last+1):
            itNameIdx = self.index(itNum, 0, parent)
            item = self.itemFromIndex(itNameIdx)

            if type(item) is IPFSNameItem:
                self.entryCache.register(item.entry)

    def onRowsToBeRemoved(self, parent, first, last):
        for itNum in range(first, last+1):
            itNameIdx = self.index(itNum, 0, parent)
            item = self.itemFromIndex(itNameIdx)

            if type(item) is IPFSNameItem:
                entry = item.entry
                self.entryCache.purge(entry['Hash'])

    def supportedDropActions(self):
        return Qt.CopyAction | Qt.MoveAction | Qt.TargetMoveAction | Qt.LinkAction

    def mimeData(self, indexes):
        mimedata = QMimeData()
        return mimedata

    def canDropMimeData(self, data, action, row, column, parent):
        mimeText = data.text()

        if mimeText and mimeText.startswith('file://'):
            return True
        return False

    def dropMimeData(self, data, action, row, column, parent):
        mimeText = data.text()
        itemIdx = self.index(row, column, parent)
        item = self.itemFromIndex(itemIdx)

        if data.hasUrls():
            for url in data.urls():
                path = url.toLocalFile()
                if os.path.isfile(path):
                    self.fileDropEvent.emit(path)
                if os.path.isdir(path):
                    self.directoryDropEvent.emit(path)
            return True
        else:
            return False

    def getHashFromIdx(self, idx):
        idxHash = self.index(idx.row(), 2, idx.parent())
        return self.data(idxHash)

    def getNameFromIdx(self, idx):
        idxName = self.index(idx.row(), 0, idx.parent())
        return self.data(idxName)

    def getNameItemFromIdx(self, idx):
        idxName = self.index(idx.row(), 0, idx.parent())
        return self.itemFromIndex(idxName)

def makeFilesModel():
    # Setup the model
    model = FilesItemModel()
    model.setHorizontalHeaderLabels(
            [iFileName(), iFileSize(), iFileHash()])
    return model

class FilesTab(GalacteekTab):
    statusReady = 0
    statusBusy = 1

    def __init__(self, gWindow, **kw):
        super().__init__(gWindow, **kw)

        self.lock = asyncio.Lock()

        self.ui = ui_files.Ui_FileManagerForm()
        self.ui.setupUi(self)
        self.clipboard = self.app.appClipboard
        self.status = self.statusReady

        # Build file browser
        self.createFileManager()

        # Connect the various buttons
        self.ui.addFileButton.clicked.connect(self.onAddFilesClicked)
        self.ui.addDirectoryButton.clicked.connect(self.onAddDirClicked)
        self.ui.refreshButton.clicked.connect(self.onRefreshClicked)
        self.ui.searchFiles.returnPressed.connect(self.onSearchFiles)
        self.ui.fileManagerSwitch.clicked.connect(self.onFileManager)
        self.ui.fileManagerButton.clicked.connect(self.onFileManager)

        # Connect the tree view actions
        self.ui.treeFiles.doubleClicked.connect(self.onDoubleClicked)
        self.ui.treeFiles.setContextMenuPolicy(Qt.CustomContextMenu)
        self.ui.treeFiles.customContextMenuRequested.connect(self.onContextMenu)
        self.ui.treeFiles.expanded.connect(self.onExpanded)

        # Path selector
        self.ui.pathSelector.insertItem(0, getIcon('go-home.png'), 'Home')
        self.ui.pathSelector.insertItem(1, getIcon('folder-pictures.png'),
                iPictures())
        self.ui.pathSelector.insertItem(2, getIcon('folder-videos.png'),
                iVideos())
        self.ui.pathSelector.insertItem(3, getIcon('folder-music.png'),
                iMusic())
        self.ui.pathSelector.insertItem(4, getIcon('code-fork.png'), iCode())
        self.ui.pathSelector.insertItem(5, getIcon('folder-documents.png'),
                iDocuments())
        self.ui.pathSelector.activated.connect(self.onPathSelector)

        # Connect the event filter
        evfilter = IPFSTreeKeyFilter(self.ui.treeFiles)
        evfilter.copyHashPressed.connect(self.onCopyItemHash)
        evfilter.copyPathPressed.connect(self.onCopyItemPath)
        evfilter.returnPressed.connect(self.onReturn)
        evfilter.explorePressed.connect(self.onExploreItem)
        self.ui.treeFiles.installEventFilter(evfilter)

        self.setupModel()

        # Setup the tree view
        self.ui.treeFiles.setExpandsOnDoubleClick(True)
        self.ui.treeFiles.setItemsExpandable(True)
        self.ui.treeFiles.setSortingEnabled(True)
        self.ui.treeFiles.sortByColumn(0, Qt.AscendingOrder)

        if self.app.settingsMgr.hideHashes:
            self.ui.treeFiles.hideColumn(2)

        self.iconFolder = getIcon('folder-open.png')
        self.iconFile = getIcon('file.png')

        # Configure drag-and-drop
        self.ui.treeFiles.setAcceptDrops(True)
        self.ui.treeFiles.setDragDropMode(QAbstractItemView.DropOnly)

        self.ipfsKeys = []

    @property
    def displayPath(self):
        return self.displayItem.getPath()

    @property
    def busy(self):
        return self.status == self.statusBusy

    def createFileManager(self):
        self.fManagerModel = QFileSystemModel()
        self.fManagerModel.setRootPath('')

        self.localTree = QTreeView()
        self.localTree.setModel(self.fManagerModel)
        self.localTree.setDragEnabled(True)
        self.localTree.setDragDropMode(QAbstractItemView.DragOnly)
        self.localTree.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.localTree.setItemsExpandable(True)

        rootIndex = self.fManagerModel.index(QDir.rootPath())
        if rootIndex.isValid():
            self.localTree.setRootIndex(rootIndex)

        for col in range(1, 4):
            self.localTree.hideColumn(col)

        self.localTree.hide()
        self.ui.hLayoutBrowser.insertWidget(0, self.localTree)

    def setupModel(self):
        # Setup the model, caching it in the profile object
        if self.profile.filesModel:
            self.model = self.profile.filesModel
        else:
            self.model = makeFilesModel()
            self.profile.setFilesModel(self.model)
            self.model.setupItemsFromProfile(self.profile)

        self.ui.treeFiles.setModel(self.model)
        self.changeDisplayItem(self.model.itemHome)
        self.app.task(self.updateKeys)

        self.disconnectDropSignals()

        # Connect the model's drag-and-drop signals
        self.model.fileDropEvent.connect(self.onDropFile)
        self.model.directoryDropEvent.connect(self.onDropDirectory)

    def enableButtons(self, flag=True):
        for btn in [ self.ui.addFileButton,
                self.ui.addDirectoryButton,
                self.ui.refreshButton,
                self.ui.fileManagerSwitch,
                self.ui.fileManagerButton,
                self.ui.pathSelector,
                self.ui.searchFiles ]:
            btn.setEnabled(flag)

    def currentItem(self):
        currentIdx = self.ui.treeFiles.currentIndex()
        if currentIdx.isValid():
            return self.model.getNameItemFromIdx(currentIdx)

    def onClose(self):
        if not self.busy:
            self.disconnectDropSignals()
            return True
        return False

    def disconnectDropSignals(self):
        try:
            self.model.fileDropEvent.disconnect(self.onDropFile)
            self.model.directoryDropEvent.disconnect(self.onDropDirectory)
        except Exception as e:
            pass

    def onFileManager(self):
        if self.localTree.isHidden():
            self.localTree.show()
        else:
            self.localTree.hide()

    def onDropFile(self, path):
        self.scheduleAddFiles([path])

    def onDropDirectory(self, path):
        self.scheduleAddDirectory(path)

    def onCopyItemHash(self):
        currentItem = self.currentItem()
        if currentItem:
            dataHash = self.model.getHashFromIdx(currentItem.index())
            self.app.setClipboardText(dataHash)

    def onCopyItemPath(self):
        currentItem = self.currentItem()
        if currentItem:
            dataHash = self.model.getHashFromIdx(currentItem.index())
            self.app.setClipboardText(joinIpfs(dataHash))

    def onReturn(self):
        currentItem = self.currentItem()
        dataHash = self.model.getHashFromIdx(currentItem.index())
        if dataHash:
            self.gWindow.addBrowserTab().browseIpfsHash(dataHash)

    def onExpanded(self, idx):
        pass

    def pathSelectorDefault(self):
        self.ui.pathSelector.setCurrentIndex(0)

    def onPathSelector(self, idx):
        if self.busy:
            return

        text = self.ui.pathSelector.itemText(idx)

        if text == iHome():
            self.changeDisplayItem(self.model.itemHome)
        if text == iPictures():
            self.changeDisplayItem(self.model.itemPictures)
        if text == iVideos():
            self.changeDisplayItem(self.model.itemVideos)
        if text == iMusic():
            self.changeDisplayItem(self.model.itemMusic)
        if text == iCode():
            self.changeDisplayItem(self.model.itemCode)
        if text == iDocuments():
            self.changeDisplayItem(self.model.itemDocuments)

    def changeDisplayItem(self, item):
        self.displayItem = item
        self.ui.treeFiles.setRootIndex(self.displayItem.index())
        self.updateTree()
        self.ui.treeFiles.expand(self.displayItem.index())

    def onContextMenuVoid(self, point):
        idx = self.ui.treeFiles.indexAt(point)
        menu = QMenu()
        menu.exec(self.ui.treeFiles.mapToGlobal(point))

    def onContextMenu(self, point):
        idx = self.ui.treeFiles.indexAt(point)
        if not idx.isValid():
            return self.onContextMenuVoid(point)

        nameItem = self.model.getNameItemFromIdx(idx)
        dataHash = self.model.getHashFromIdx(idx)
        dataPath = self.model.getNameFromIdx(idx)
        ipfsPath = joinIpfs(dataHash)
        menu = QMenu()

        def unlink(hash):
            self.scheduleUnlink(hash)

        def explore(hash):
            self.gWindow.exploreHash(hash)

        def delete(hash):
            self.scheduleDelete(hash)

        def hashmark(mPath, name):
            addHashmark(self.app.marksLocal, mPath, name)

        def browse(hash):
            self.browse(hash)

        def copyHashToClipboard(itemHash, clipboardType):
            self.clipboard.setText(itemHash, clipboardType)

        def openWithMediaPlayer(itemHash):
            parentHash = nameItem.getParentHash()
            name = nameItem.entry['Name']
            if parentHash:
                fp = joinIpfs(os.path.join(parentHash, name))
                self.gWindow.mediaPlayerPlay(fp, mediaName=name)
            else:
                self.gWindow.mediaPlayerPlay(joinIpfs(itemHash),
                        mediaName = name)

        menu.addAction(iCopyHashToSelClipboard(), lambda:
            copyHashToClipboard(dataHash, QClipboard.Selection))
        menu.addAction(iCopyHashToGlobalClipboard(), lambda:
            copyHashToClipboard(dataHash, QClipboard.Clipboard))
        menu.addAction(iUnlinkFile(), lambda:
            unlink(dataHash))
        menu.addAction(iDeleteFile(), lambda:
            delete(dataHash))
        menu.addAction(iHashmarkFile(), lambda:
            hashmark(ipfsPath, nameItem.entry['Name']))
        menu.addAction(iBrowseFile(), lambda:
            browse(dataHash))

        if nameItem.isDir():
            menu.addAction(iExploreDir(), lambda:
                explore(dataHash))

        def publishToKey(action):
            key = action.data()['key']['Name']
            oHash = action.data()['hash']

            async def publish(op, oHash, keyName):
                r = await op.publish(joinIpfs(oHash), key=keyName)

            self.app.ipfsTaskOp(publish, oHash, key)

        # Populate publish menu
        publishMenu = QMenu('Publish to IPFS key')
        for key in self.ipfsKeys:
            action = QAction(key['Name'], self)
            action.setData({
                'key': key,
                'hash': dataHash
            })

            publishMenu.addAction(action)

        publishMenu.triggered.connect(publishToKey)

        openWithMenu = QMenu(iOpenWith())
        openWithMenu.addAction(iMediaPlayer(), lambda:
                openWithMediaPlayer(dataHash))

        menu.addMenu(publishMenu)
        menu.addMenu(openWithMenu)
        menu.exec(self.ui.treeFiles.mapToGlobal(point))

    def browse(self, hash):
        self.gWindow.addBrowserTab().browseIpfsHash(hash)

    def browseFs(self, path):
        self.gWindow.addBrowserTab().browseFsPath(path)

    def onExploreItem(self):
        current = self.currentItem()

        if current and current.isDir():
            dataHash = self.model.getHashFromIdx(current.index())
            self.gWindow.exploreHash(dataHash)

    def onDoubleClicked(self, idx):
        if not idx.isValid():
            return

        nameItem = self.model.getNameItemFromIdx(idx)
        item = self.model.itemFromIndex(idx)
        dataHash = self.model.getHashFromIdx(idx)
        dataPath = self.model.getNameFromIdx(idx)

        if nameItem.isFile():
            fileName = nameItem.text()
            finalPath = joinIpfs(dataHash)

            # Find the parent hash
            parentHash = nameItem.getParentHash()
            if parentHash:
                # We have the parent hash, so use it to build a file path
                # preserving the real file name
                finalPath = joinIpfs(os.path.join(parentHash, fileName))

            if nameItem.mimeType:
                cat = nameItem.mimeCategory
                # If it's media content try to open it in the media player
                if cat and (cat == 'audio' or cat == 'video'):
                    return self.gWindow.mediaPlayerPlay(finalPath,
                            mediaName=fileName)

            return self.browseFs(finalPath)

        self.app.task(self.listFiles, item.getPath(), parentItem=item,
            autoexpand=True)

    def onSearchFiles(self):
        search = self.ui.searchFiles.text()
        self.ui.treeFiles.keyboardSearch(search)

    def onMkDirClicked(self):
        dirName = QInputDialog.getText(self, 'Directory name',
                'Directory name')
        if dirName:
            self.app.task(self.makeDir, self.displayItem.getPath(),
                    dirName[0])

    def onRefreshClicked(self):
        self.updateTree()
        self.ui.treeFiles.setFocus(Qt.OtherFocusReason)

    def onAddDirClicked(self):
        result = QFileDialog.getExistingDirectory(None,
            iSelectDirectory(), getHomePath(),
            QFileDialog.ShowDirsOnly)
        if result:
            self.scheduleAddDirectory(result)

    def statusAdded(self, name):
        self.statusSet(iAddedFile(name))

    def statusLoading(self, name):
        self.statusSet(iLoading(name))

    def statusSet(self, msg):
        self.ui.statusLabel.setText(msg)

    def onAddFilesClicked(self):
        self.addFilesDialog()

    def addFilesDialog(self, parent=None):
        if parent is None:
            parent = self.displayPath

        result = QFileDialog.getOpenFileNames(None,
            iSelectFiles(), getHomePath(), '(*.*)')
        if not result:
            return

        self.scheduleAddFiles(result[0], parent=parent)

    def scheduleAddFiles(self, path, parent=None):
        if self.busy:
            return
        if parent is None:
            parent = self.displayPath
        return self.app.task(self.addFiles, path, parent)

    def scheduleAddDirectory(self, path):
        if self.busy:
            return
        return self.app.task(self.addDirectory, path)

    def scheduleUnlink(self, hash):
        return self.app.task(self.unlinkFileFromHash, hash)

    def scheduleDelete(self, hash):
        return self.app.task(self.deleteFromHash, hash)

    def updateTree(self):
        self.app.task(self.listFiles, self.displayPath,
            parentItem=self.displayItem, maxdepth=1)

    @ipfsOp
    async def updateKeys(self, ipfsop):
        self.ipfsKeys = await ipfsop.keys()

    @ipfsOp
    async def makeDir(self, ipfsop, parent, path):
        ret = await ipfsop.filesMkdir(os.path.join(parent, path))
        self.updateTree()

    @ipfsOp
    async def listFiles(self, ipfsop, path, parentItem, maxdepth=0,
            autoexpand=False):
        if self.busy:
            return
        self.enableButtons(flag=False)
        self.status = self.statusBusy

        try:
            await asyncio.wait_for(
                self.listPath(ipfsop, path, parentItem=parentItem,
                    maxdepth=maxdepth, autoexpand=autoexpand), 60)
        except aioipfs.APIError:
            messageBox(iErrNoCx())

        self.enableButtons()
        self.status = self.statusReady

    async def listPath(self, op, path, parentItem=None, depth=0, maxdepth=1,
            autoexpand=False):
        if not parentItem.getPath():
            return

        listing = await op.filesList(path)
        if not listing:
            return

        parentItemSibling = self.model.sibling(parentItem.row(), 2,
                parentItem.index())
        parentItemHash = self.model.data(parentItemSibling)

        for entry in listing:
            await asyncio.sleep(0)

            if entry['Hash'] == '' or entry['Hash'] in self.model.entryCache:
                continue

            if entry['Type'] == 1: # directory
                icon = self.iconFolder
            else:
                icon = self.iconFile

            nItemName = IPFSNameItem(entry, entry['Name'], icon)
            nItemName.mimeFromDb(self.app.mimeDb)
            nItemName.setParentHash(parentItemHash)
            nItemSize = IPFSItem(sizeFormat(entry['Size']))
            nItemHash = IPFSItem(entry['Hash'])

            nItemName.setPath(os.path.join(parentItem.getPath(),
                entry['Name']))

            nItem = [nItemName, nItemSize, nItemHash]

            parentItem.appendRow(nItem)

            if entry['Type'] == 1: # directory
                if autoexpand is True:
                    self.ui.treeFiles.setExpanded(nItemName.index(), True)
                await asyncio.sleep(0)
                if maxdepth > depth:
                    depth += 1
                    await self.listPath(op,
                        nItemName.getPath(),
                        parentItem=nItemName,
                        maxdepth=maxdepth, depth=depth)
                    depth -= 1

        if autoexpand is True:
            self.ui.treeFiles.expand(parentItem.index())

    @ipfsOp
    async def deleteFromHash(self, ipfsop, hash):
        entry = await ipfsop.filesLookupHash(self.displayPath, hash)

        if entry:
            purged = await ipfsop.purge(hash)
            await ipfsop.filesDelete(self.displayPath,
                entry['Name'], recursive=True)
            await modelhelpers.modelDeleteAsync(self.model, hash)

    @ipfsOp
    async def unlinkFileFromHash(self, op, hash):
        listing = await op.filesList(self.displayPath)
        for entry in listing:
            if entry['Hash'] == hash:
                await op.filesDelete(self.displayPath,
                    entry['Name'], recursive=True)
                await modelhelpers.modelDeleteAsync(self.model, hash)

    @ipfsOp
    async def addFiles(self, op, files, parent):
        """ Add every file with an optional wrapper directory """

        wrapEnabled = self.app.settingsMgr.isTrue(
            CFG_SECTION_UI, CFG_KEY_WRAPSINGLEFILES)

        self.enableButtons(flag=False)
        self.status = self.statusBusy
        last = None

        for file in files:
            async def onEntry(entry):
                self.statusAdded(entry['Name'])

            root = await op.addPath(file, wrap=wrapEnabled,
                    callback=onEntry)

            if root is None:
                self.statusSet(iFileImportError())
                continue

            base = os.path.basename(file)
            if wrapEnabled is True:
                base += '.dirw'

            await self.linkEntry(op, root, parent, base)
            last = root['Hash']

        self.enableButtons()
        self.status = self.statusReady
        self.updateTree()
        return True

    @ipfsOp
    async def addDirectory(self, op, path):
        wrapEnabled = self.app.settingsMgr.isTrue(
            CFG_SECTION_UI, CFG_KEY_WRAPDIRECTORIES)
        self.enableButtons(flag=False)
        self.status = self.statusBusy
        basename = os.path.basename(path)
        dirEntry = None

        async def onEntry(entry):
            self.statusAdded(entry['Name'])

        dirEntry = await op.addPath(path, callback=onEntry,
                recursive=True, wrap=wrapEnabled)

        if not dirEntry:
            # Nothing went through ?
            self.enableButtons()
            return False

        if wrapEnabled is True:
            basename += '.dirw'

        await self.linkEntry(op, dirEntry, self.displayPath, basename)

        self.enableButtons()
        self.status = self.statusReady
        self.updateTree()
        return True

    async def linkEntry(self, op, entry, dest, basename):
        for lIndex in range(0, 16):
            await asyncio.sleep(0)
            if lIndex == 0:
                lNew = basename
            else:
                lNew = '{0}.{1}'.format(basename, lIndex)
            lookup = await op.filesLookup(dest, lNew)
            if not lookup:
                linkS = await op.filesLink(entry, dest, name=lNew)
                if linkS:
                    return lNew



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/profilepostmessage.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_PostMessageDialog(object):
    def setupUi(self, PostMessageDialog):
        PostMessageDialog.setObjectName("PostMessageDialog")
        PostMessageDialog.resize(511, 319)
        self.verticalLayout = QtWidgets.QVBoxLayout(PostMessageDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.message = QtWidgets.QTextEdit(PostMessageDialog)
        self.message.setObjectName("message")
        self.gridLayout.addWidget(self.message, 3, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(PostMessageDialog)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)
        self.label = QtWidgets.QLabel(PostMessageDialog)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 2, 0, 1, 1)
        self.title = QtWidgets.QLineEdit(PostMessageDialog)
        self.title.setMaxLength(256)
        self.title.setObjectName("title")
        self.gridLayout.addWidget(self.title, 1, 0, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(PostMessageDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(PostMessageDialog)
        self.buttonBox.accepted.connect(PostMessageDialog.accept)
        self.buttonBox.rejected.connect(PostMessageDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(PostMessageDialog)

    def retranslateUi(self, PostMessageDialog):
        _translate = QtCore.QCoreApplication.translate
        PostMessageDialog.setWindowTitle(_translate("PostMessageDialog", "Post a message"))
        self.label_2.setText(_translate("PostMessageDialog", "Title"))
        self.label.setText(_translate("PostMessageDialog", "Message"))





import os.path

from PyQt5.QtWidgets import (QWidget, QFrame, QApplication, QMainWindow,
        QDialog, QLabel, QTextEdit, QPushButton, QVBoxLayout, QAction,
        QHBoxLayout, QTabWidget, QFileDialog, QListView, QSplitter,
        QToolButton, QStyle, QSlider, QGraphicsScene, QMenu)

from PyQt5.QtMultimedia import (QMediaPlayer, QMediaContent, QMediaPlaylist,
    QMediaMetaData)
from PyQt5.QtMultimediaWidgets import QVideoWidget, QGraphicsVideoItem

from PyQt5.QtCore import (QCoreApplication, QUrl, Qt, QAbstractItemModel,
    QModelIndex, QTime, QSize)

from galacteek.core.jsono import *
from galacteek.ipfs.ipfsops import *

from . import ui_mediaplayer, ui_mediaplaylist

from .widgets import *
from .helpers import *

def iPlayerError(code):
    return QCoreApplication.translate('MediaPlayer',
            'Media player error (code: {0})').format(code)


def iFullScreen():
    return QCoreApplication.translate('MediaPlayer', 'Fullscreen')

def iCopyPlaylistPath():
    return QCoreApplication.translate('MediaPlayer',
            "Copy playlist's IPFS path to the clipboard")

def iLoadPlaylistFromPath():
    return QCoreApplication.translate('MediaPlayer',
            'Load playlist from the clipboard')

def iCannotLoadPlaylist():
    return QCoreApplication.translate('MediaPlayer',
            'Cannot load playlist')

def iPlaylistExists():
    return QCoreApplication.translate('MediaPlayer',
            'A playlist with this name already exists')

def iPlaylistName():
    return QCoreApplication.translate('MediaPlayer',
            'Playlist name')

def iAlreadyInPlaylist():
    return QCoreApplication.translate('MediaPlayer',
            'Already queued in the current playlist')

class VideoWidget(QVideoWidget):
    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Escape:
            self.setFullScreen(False)

        super(VideoWidget, self).keyPressEvent(event)

def durationConvert(duration):
    return QTime((duration/3600)%60, (duration/60)%60,
        duration%60, (duration*1000)%1000)

class JSONPlaylistV1(QJSONObj):
    """
    V1 JSON playlist document
    """
    def prepare(self, root):
        root['playlist'] = {
            'name': self.listName,
            'formatv': 1,
            'items': []
        }

        for path in self.itemPaths:
            root['playlist']['items'].append({
                'path': path
            })
        self.changed.emit()

    def items(self):
        return self.root['playlist']['items']

class MediaPlayerTab(GalacteekTab):
    statePlaying  = QMediaPlayer.PlayingState
    statePaused   = QMediaPlayer.PausedState
    stateStopped  = QMediaPlayer.StoppedState

    def __init__(self, *args, **kw):
        super(MediaPlayerTab, self).__init__(*args, **kw)

        self.ui = ui_mediaplayer.Ui_MediaPlayer()
        self.ui.setupUi(self)

        self.playlistIpfsPath = None
        self.playlist = QMediaPlaylist()
        self.model = ListModel(self.playlist)

        self.playlistsMenu = QMenu()
        self.playlistsMenu.triggered.connect(self.onPlaylistsMenu)

        self.pListWidget = QWidget(self)
        self.uipList = ui_mediaplaylist.Ui_MediaPlaylist()
        self.uipList.setupUi(self.pListWidget)
        self.uipList.savePlaylistButton.clicked.connect(self.onSavePlaylist)
        self.uipList.loadPlaylistButton.setPopupMode(QToolButton.MenuButtonPopup)
        self.uipList.loadPlaylistButton.setMenu(self.playlistsMenu)

        self.clipMenu = QMenu()
        self.copyPathAction = QAction(getIconIpfsIce(),
                iCopyPlaylistPath(), self,
                triggered=self.onCopyPlaylistPath)
        self.loadPathAction = QAction(getIconIpfsIce(),
                iLoadPlaylistFromPath(), self,
                triggered=self.onLoadPlaylistPath)

        self.copyPathAction.setEnabled(False)
        self.loadPathAction.setEnabled(self.app.clipTracker.hasIpfs)
        self.clipMenu.addAction(self.copyPathAction)
        self.clipMenu.addAction(self.loadPathAction)

        self.uipList.clipPlaylistButton.setPopupMode(QToolButton.MenuButtonPopup)
        self.uipList.clipPlaylistButton.setMenu(self.clipMenu)
        self.uipList.clipPlaylistButton.clicked.connect(self.onLoadPlaylistPath)
        self.uipList.clearButton.clicked.connect(self.onClearPlaylist)

        self.uipList.nextButton.clicked.connect(self.onPlaylistNext)
        self.uipList.previousButton.clicked.connect(self.onPlaylistPrevious)
        self.uipList.nextButton.setIcon(
                self.style().standardIcon(QStyle.SP_MediaSkipForward))
        self.uipList.previousButton.setIcon(
                self.style().standardIcon(QStyle.SP_MediaSkipBackward))

        self.pListView = self.uipList.listView
        self.pListView.setModel(self.model)
        self.pListView.setResizeMode(QListView.Adjust)
        self.pListView.setMinimumWidth(self.width() / 2)

        self.duration = None
        self.playerState = None
        self.player = QMediaPlayer()
        self.player.setPlaylist(self.playlist)

        self.videoWidget = VideoWidget()

        self.player.setVideoOutput(self.videoWidget)

        self.player.error.connect(self.onError)
        self.player.stateChanged.connect(self.onStateChanged)
        self.player.metaDataChanged.connect(self.onMetaData)
        self.playlist.currentIndexChanged.connect(self.playlistPositionChanged)
        self.player.durationChanged.connect(self.mediaDurationChanged)
        self.player.positionChanged.connect(self.mediaPositionChanged)
        self.pListView.activated.connect(self.onListActivated)

        self.togglePList = QToolButton()
        self.togglePList.setIcon(self.style().standardIcon(QStyle.SP_ArrowRight))
        self.togglePList.setFixedSize(32, 128)
        self.togglePList.clicked.connect(self.onTogglePlaylist)

        self.clipboardButton = QToolButton(clicked=self.onClipboardClicked)
        self.clipboardButton.setIcon(getIconClipboard())
        self.clipboardButton.setEnabled(self.app.clipTracker.hasIpfs)
        self.app.clipTracker.clipboardHasIpfs.connect(self.onClipboardIpfs)

        self.playButton = QToolButton(clicked=self.onPlayClicked)
        self.playButton.setIcon(self.style().standardIcon(QStyle.SP_MediaPlay))

        self.pauseButton = QToolButton(clicked=self.onPauseClicked)
        self.pauseButton.setIcon(self.style().standardIcon(QStyle.SP_MediaPause))
        self.pauseButton.setEnabled(False)

        self.stopButton = QToolButton(clicked=self.onStopClicked)
        self.stopButton.setIcon(self.style().standardIcon(QStyle.SP_MediaStop))
        self.stopButton.setEnabled(False)

        self.fullscreenButton = QToolButton(clicked=self.onFullScreen)
        self.fullscreenButton.setIcon(getIcon('fullscreen.png'))
        self.fullscreenButton.setToolTip(iFullScreen())

        self.seekSlider = QSlider(Qt.Horizontal, sliderMoved=self.onSeek)
        self.durationLabel = QLabel()

        vLayout = QVBoxLayout()
        hLayoutControls = QHBoxLayout()
        hLayoutControls.setContentsMargins(0, 0, 0, 0)
        hLayoutControls.addWidget(self.clipboardButton)
        hLayoutControls.addWidget(self.playButton)
        hLayoutControls.addWidget(self.pauseButton)
        hLayoutControls.addWidget(self.stopButton)
        hLayoutControls.addWidget(self.seekSlider)
        hLayoutControls.addWidget(self.durationLabel)
        hLayoutControls.addWidget(self.fullscreenButton)
        vLayout.addWidget(self.videoWidget)
        vLayout.addLayout(hLayoutControls)

        hLayout = QHBoxLayout()
        hLayout.addLayout(vLayout)
        hLayout.addWidget(self.pListWidget)
        hLayout.addWidget(self.togglePList)

        self.pListWidget.hide()

        self.ui.verticalLayout.addLayout(hLayout)
        self.update()

    @property
    def isPlaying(self):
        return self.playerState == self.statePlaying

    @property
    def isPaused(self):
        return self.playerState == self.statePaused

    @property
    def isStopped(self):
        return self.playerState == self.stateStopped

    def update(self):
        self.app.task(self.updatePlaylistsMenu)

    def onFullScreen(self):
        self.videoWidget.setFullScreen(True)

    def onClearPlaylist(self):
        self.copyPathAction.setEnabled(False)
        self.player.stop()
        self.clearPlaylist()

    def onLoadPlaylistPath(self):
        current = self.app.clipTracker.getCurrent()
        if current:
            self.app.task(self.loadPlaylistFromPath, current['path'])

    def onCopyPlaylistPath(self):
        if self.playlistIpfsPath:
            self.app.setClipboardText(self.playlistIpfsPath)

    @ipfsOp
    async def updatePlaylistsMenu(self, ipfsop):
        currentList = [action.text() for action in \
                self.playlistsMenu.actions()]
        listing = await ipfsop.filesList(self.profile.pathPlaylists)
        for entry in listing:
            if entry['Name'] in currentList:
                continue
            action = QAction(entry['Name'], self)
            action.setData(entry)
            self.playlistsMenu.addAction(action)

    def onPlaylistsMenu(self, action):
        entry = action.data()
        self.app.task(self.loadPlaylistFromPath, joinIpfs(entry['Hash']))

    def onSavePlaylist(self):
        paths = self.playlistGetPaths()
        listName = inputText(title=iPlaylistName(), label=iPlaylistName())
        if not listName:
            return

        obj = JSONPlaylistV1(listName=listName, itemPaths=paths)
        self.app.task(self.savePlaylist, obj, listName)

    @ipfsOp
    async def savePlaylist(self, ipfsop, obj, name):
        objPath = os.path.join(self.profile.pathPlaylists, name)
        exists = await ipfsop.filesStat(objPath)

        if exists:
            return messageBox(iPlaylistExists())

        ent = await ipfsop.client.core.add_json(obj.root)

        if ent:
            ret = await ipfsop.filesLinkFp(ent, objPath)
            self.playlistIpfsPath = joinIpfs(ent['Hash'])
            self.copyPathAction.setEnabled(True)

        self.update()

    @ipfsOp
    async def loadPlaylistFromPath(self, ipfsop, path):
        obj = await ipfsop.jsonLoad(path)

        if obj is None:
            return messageBox(iCannotLoadPlaylist())

        try:
            # Assume v1 format for now, when the format evolves we'll just
            # use json validation
            pList = JSONPlaylistV1(data=obj)
            self.clearPlaylist()

            for item in pList.items():
                self.queueFromPath(item['path'])

            self.playlistIpfsPath = path
            self.copyPathAction.setEnabled(True)
        except Exception as e:
            return messageBox(iCannotLoadPlaylist())

    def playlistGetPaths(self):
        return [u.path() for u in self.playlistGetUrls() ]

    def playlistGetUrls(self):
        urls = []
        for idx in range(0, self.playlist.mediaCount()):
            media = self.playlist.media(idx)
            urls.append(media.canonicalUrl())
        return urls

    def onClipboardIpfs(self, valid, cid, path):
        self.clipboardButton.setEnabled(valid)
        self.clipboardButton.setToolTip(path)
        self.loadPathAction.setEnabled(valid)

    def onClipboardClicked(self):
        current = self.app.clipTracker.getCurrent()
        if current:
            self.playFromPath(current['path'])

    def onPlaylistNext(self):
        self.playlist.next()

    def onPlaylistPrevious(self):
        self.playlist.previous()

    def onPlayClicked(self):
        self.player.play()

    def onPauseClicked(self):
        if self.isPlaying:
            self.player.pause()
        elif self.isPaused:
            self.player.play()

    def onStopClicked(self):
        self.player.stop()
        self.player.setPosition(0)
        self.seekSlider.setValue(0)
        self.seekSlider.setRange(0, 0)

    def onSeek(self, seconds):
        self.player.setPosition(seconds * 1000)

    def onTogglePlaylist(self):
        if self.pListWidget.isHidden():
            self.pListWidget.show()
        else:
            self.pListWidget.hide()

    def onError(self, error):
        messageBox(iPlayerError(error))

    def onStateChanged(self, state):
        self.playerState = state
        self.updateControls(state)

    def updateControls(self, state):
        if self.isStopped:
            self.stopButton.setEnabled(False)
            self.pauseButton.setEnabled(False)
            self.playButton.setEnabled(True)
            self.seekSlider.setEnabled(False)
            self.duration = None
        elif self.isPlaying:
            self.seekSlider.setRange(0, self.player.duration() / 1000)
            self.seekSlider.setEnabled(True)
            self.pauseButton.setEnabled(True)
            self.playButton.setEnabled(False)
            self.stopButton.setEnabled(True)

    def onListActivated(self, index):
        if index.isValid():
            self.playlist.setCurrentIndex(index.row())
            self.player.play()

    def onMetaData(self):
        # Unfinished
        if self.player.isMetaDataAvailable():
            availableKeys = self.player.availableMetaData()

            for key in availableKeys:
                value = self.player.metaData(key)

    def playFromUrl(self, url, mediaName=None):
        if self.isPlaying:
            self.player.stop()

        cUrls = self.playlistGetUrls()
        for u in cUrls:
            if u.toString() == url.toString():
                return messageBox(iAlreadyInPlaylist())

        media = QMediaContent(url)
        if self.playlist.addMedia(media):
            count = self.model.rowCount()
            if count > 0:
                self.playlist.setCurrentIndex(count - 1)

        self.player.play()

    def playFromPath(self, path, mediaName=None):
        mediaUrl = self.app.subUrl(path)
        self.playFromUrl(mediaUrl)

    def queueFromPath(self, path, mediaName=None):
        mediaUrl = self.app.subUrl(path)
        self.playlist.addMedia(QMediaContent(mediaUrl))

    def clearPlaylist(self):
        self.playlist.clear()
        self.pListView.reset()

    def playlistPositionChanged(self, position):
        self.pListView.setCurrentIndex(self.model.index(position, 0))

    def mediaDurationChanged(self, duration):
        self.duration = duration / 1000
        self.seekSlider.setMaximum(self.duration)

    def mediaPositionChanged(self, progress):
        progress /= 1000

        if self.duration:
            cTime = durationConvert(progress)
            tTime = durationConvert(self.duration)
            self.durationLabel.setText('{0} ({1})'.format(
                cTime.toString(), tTime.toString()))

        if not self.seekSlider.isSliderDown():
            self.seekSlider.setValue(progress)

    def onClose(self):
        self.player.stop()
        self.player.setMedia(QMediaContent(None))
        return True

    def keyPressEvent(self, event):
        modifiers = event.modifiers()
        mSecMove = 3000

        if event.key() == Qt.Key_Space:
            if self.isPlaying:
                self.player.pause()
            elif self.isPaused:
                self.player.play()
        if event.key() == Qt.Key_Right:
            pos = self.player.position()
            self.player.setPosition(pos + mSecMove)
        if event.key() == Qt.Key_Up:
            pos = self.player.position()
            self.player.setPosition(pos + mSecMove*2)
        if event.key() == Qt.Key_Left:
            pos = self.player.position()
            if pos > mSecMove:
                self.player.setPosition(pos - mSecMove)
            else:
                self.player.setPosition(0)
        if event.key() == Qt.Key_Down:
            pos = self.player.position()
            if pos > mSecMove*2:
                self.player.setPosition(pos - mSecMove*2)
            else:
                self.player.setPosition(0)
        if event.key() == Qt.Key_F:
            self.videoWidget.setFullScreen(True)

        super(MediaPlayerTab, self).keyPressEvent(event)

class ListModel(QAbstractItemModel):
    def __init__(self, playlist, parent=None):
        super(ListModel, self).__init__(parent)
        self.playlist = playlist
        self.playlist.mediaAboutToBeInserted.connect(
                self.beginInsertItems)
        self.playlist.mediaInserted.connect(self.endInsertItems)
        self.playlist.mediaChanged.connect(self.changeItems)

    def rowCount(self, parent=QModelIndex()):
        return self.playlist.mediaCount()

    def columnCount(self, parent=QModelIndex()):
        return 1 if not parent.isValid() else 0

    def index(self, row, column, parent=QModelIndex()):
        return self.createIndex(row, column)

    def parent(self, child):
        return QModelIndex()

    def beginInsertItems(self, start, end):
        self.beginInsertRows(QModelIndex(), start, end)

    def endInsertItems(self):
        self.endInsertRows()

    def changeItems(self, start, end):
        self.dataChanged.emit(self.index(start, 0), self.index(end, 1))

    def data(self, index, role=Qt.DisplayRole):
        if index.isValid() and role == Qt.DisplayRole:
            if index.column() == 0:
                media = self.playlist.media(index.row())
                if media is None:
                    return iUnknown()
                location = media.canonicalUrl()
                path = location.path()
                basename = os.path.basename(path)
                if basename:
                    return basename
                else:
                    return path
            return self.m_data[index]
        return None



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/ipfssearchwresults.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_IPFSSearchResults(object):
    def setupUi(self, IPFSSearchResults):
        IPFSSearchResults.setObjectName("IPFSSearchResults")
        IPFSSearchResults.resize(493, 394)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(IPFSSearchResults)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.browser = QtWidgets.QTextBrowser(IPFSSearchResults)
        self.browser.setObjectName("browser")
        self.verticalLayout.addWidget(self.browser)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(IPFSSearchResults)
        QtCore.QMetaObject.connectSlotsByName(IPFSSearchResults)

    def retranslateUi(self, IPFSSearchResults):
        _translate = QtCore.QCoreApplication.translate
        IPFSSearchResults.setWindowTitle(_translate("IPFSSearchResults", "Form"))





import PyQt5.QtCore

from PyQt5.QtCore import QCoreApplication

def qtr(ctx, msg):
    return QCoreApplication.translate(ctx, msg)

# Main window messages
def iNoStatus():
    return QCoreApplication.translate('GalacteekWindow', 'No status')

def iGeneralError(msg):
    return QCoreApplication.translate('GalacteekWindow',
        'General error: {0}').format(msg)

def iErrNoCx():
    return QCoreApplication.translate('GalacteekWindow',
        'No connection available')

def iCxButNoPeers(id, agent):
    return QCoreApplication.translate('GalacteekWindow',
        'IPFS node: {0} ({1}): not connected to any peers').format(
        id, agent)

def iConnectStatus(id, agent, peerscount):
    return QCoreApplication.translate('GalacteekWindow',
        'IPFS node: {0} ({1}): connected to {2} peer(s)').format(
        id, agent, peerscount)

def iItemsInPinningQueue(items):
    return QCoreApplication.translate('GalacteekWindow',
        'Items in pinning queue: {}'.format(items))

def iUnknown():
    return QCoreApplication.translate('GalacteekWindow', 'Unknown')

def iUnknownAgent():
    return QCoreApplication.translate('GalacteekWindow', 'Unknown agent')

def iMinimized():
    return QCoreApplication.translate('GalacteekWindow',
            'Galacteek was minimized to tray')

# Headers used in the various tree widgets
def iFileName():
    return QCoreApplication.translate('IPFSTreeView', 'Name')
def iFileSize():
    return QCoreApplication.translate('IPFSTreeView', 'Size')
def iFileHash():
    return QCoreApplication.translate('IPFSTreeView', 'Hash')
def iMimeType():
    return QCoreApplication.translate('IPFSTreeView', 'Mime type')
def iUnknown():
    return QCoreApplication.translate('IPFSTreeView', 'Unknown')

def iPinSuccess(path):
    return QCoreApplication.translate('GalacteekWindow',
        '{0} was pinned successfully').format(path)

def iManual():
    return QCoreApplication.translate('GalacteekWindow', 'Manual')

def iHashmark():
    return QCoreApplication.translate('GalacteekWindow', 'Hashmark')

def iDownload():
    return QCoreApplication.translate('GalacteekWindow', 'Download')

def iMediaPlayer():
    return QCoreApplication.translate('GalacteekWindow', 'Media Player')

def iLangEnglish():
    return QCoreApplication.translate('Galacteek', 'English')

def iLangFrench():
    return QCoreApplication.translate('Galacteek', 'French')

def iYes():
    return QCoreApplication.translate('Galacteek', 'yes')

def iNo():
    return QCoreApplication.translate('Galacteek', 'no')

def iDelete():
    return QCoreApplication.translate('Galacteek', 'Delete')

def iInvalidInput():
    return QCoreApplication.translate('Galacteek', 'Invalid input')

def iKey():
    return QCoreApplication.translate('Galacteek', 'Key')

def iValue():
    return QCoreApplication.translate('Galacteek', 'Value')

def iMerkleLink():
    return QCoreApplication.translate('Galacteek', 'Merkle link')

def iIpfsInfos():
    return QCoreApplication.translate('GalacteekWindow', 'IPFS informations')

def iNoTitle():
    return QCoreApplication.translate('GalacteekWindow', 'No title')

def iFinished():
    return QCoreApplication.translate('GalacteekWindow', 'Finished')

# IPFS daemon ui messages

def iFsRepoMigrateNotFound():
    return QCoreApplication.translate('Galacteek',
        'Warning: could not find IPFS repository migration tool on your system')

def iGoIpfsNotFound():
    return QCoreApplication.translate('Galacteek',
        'Error: Could not find go-ipfs on your system')

def iGoIpfsTooOld():
    return QCoreApplication.translate('Galacteek',
        'Error: go-ipfs version found on your system is too old')

def iGoIpfsFetchAsk():
    return QCoreApplication.translate('Galacteek',
        'go-ipfs was not found on your system: download ' +
        'binary from IPFS distributions website (https://dist.ipfs.io) ?')

def iGoIpfsFetchTimeout():
    return QCoreApplication.translate('Galacteek',
        'Timeout while fetching go-ipfs distribution')

def iGoIpfsFetchSuccess():
    return QCoreApplication.translate('Galacteek',
        'go-ipfs was installed successfully')

def iGoIpfsFetchError():
    return QCoreApplication.translate('Galacteek',
        'Error while fetching go-ipfs distribution')

def iNewReleaseAvailable():
    return QCoreApplication.translate('Galacteek',
        'New release available: upgrade with pip install -U galacteek')




import sys

import aioipfs
import asyncio
import time

from galacteek import log
from galacteek.core.ipfsmarks import *
from galacteek.ipfs.ipfsops import *
from galacteek.ipfs.wrappers import ipfsOp
from galacteek.ipfs import crawl

class FeedFollower(object):
    """
    Follows an IPNS hash reference, in the sense that it will periodically
    resolve the hash and register the resulting object path if it's new,
    adding it as a mark inside the feed object
    """
    def __init__(self, app, marks):
        self.marks = marks
        self.app = app

    @ipfsOp
    async def process(self, op):
        while True:
            await asyncio.sleep(120)

            feeds = self.marks.getFeeds()

            for ipnsp, feed in feeds:
                now = int(time.time())

                resolvedLast = feed.get('resolvedlast', None)
                resolveEvery = feed.get('resolveevery', 3600)
                autoPin = feed.get('autopin', False)
                feedMarks = self.marks.getFeedMarks(ipnsp)

                if resolvedLast and resolvedLast > (now - resolveEvery):
                    continue

                resolved = await op.resolve(ipnsp)
                if not resolved:
                    continue

                resolvedPath = resolved.get('Path', None)
                if not resolvedPath:
                    continue

                feed['resolvedlast'] = now

                if resolvedPath in feedMarks.keys():
                    # already registered
                    continue

                # Register the mark
                objStats = {}
                stat = await op.objStat(resolvedPath)
                if stat:
                    objStats = stat

                title = await crawl.getTitle(op.client, resolvedPath)

                mark = IPFSHashMark.make(resolvedPath,
                        title=title,
                        datasize=objStats.get('DataSize', None),
                        cumulativesize=objStats.get('CumulativeSize', None),
                        numlinks=objStats.get('NumLinks', None),
                        share=feed['share'])

                self.marks.feedAddMark(ipnsp, mark)

                if autoPin:
                    log.debug('Feed follower, autopinning {}'.format(
                        resolvedPath))
                    await self.app.pinner.queue(resolvedPath, True, None)



import asyncio
import re
import collections
import time

from PyQt5.QtCore import (QCoreApplication, pyqtSignal, QObject)

from galacteek import log, GALACTEEK_NAME, ensure

from galacteek.ipfs import pinning
from galacteek.ipfs.wrappers import ipfsOp
from galacteek.ipfs.pubsub import *
from galacteek.ipfs.ipfsops import *
from galacteek.core.profile import UserProfile
from galacteek.core.softident import gSoftIdent
from galacteek.crypto.rsa import RSAExecutor

class PeerCtx(QObject):
    def __init__(self, ipfsCtx, peerId, identMsg, pinglast=0, pingavg=0):
        self._ipfsCtx = ipfsCtx
        self.peerId = peerId
        self.ident = identMsg
        self.pinglast = pinglast
        self.pingavg = pingavg
        self._identLast = int(time.time())

    @property
    def ipfsCtx(self):
        return self._ipfsCtx

    @property
    def ident(self):
        return self._ident

    @property
    def identLast(self):
        return self._identLast

    @property
    def peerUnresponsive(self):
        return (int(time.time()) - self.identLast) > (60*10)

    @ident.setter
    def ident(self, v):
        self._ident = v
        self._identLast = int(time.time())
        self.debug('Updated last ident to {}'.format(self.identLast))

    def debug(self, msg):
        log.debug('Peer {0}: {1}'.format(self.peerId, msg))

    @ipfsOp
    async def update(self, op):
        pass

    @ipfsOp
    async def getRsaPubKey(self, op):
        pubKeyPayload = self.ident.rsaPubKeyPem

        if pubKeyPayload:
            pubKey = await self.ipfsCtx.rsaExec.importKey(
                    pubKeyPayload)
            return pubKey
        else:
            self.debug('Failed to load pubkey')

class Peers(QObject):
    changed = pyqtSignal()
    peerAdded = pyqtSignal(str)
    peerModified = pyqtSignal(str)
    peerLogout = pyqtSignal(str)

    def __init__(self, ctx):
        super().__init__(ctx)

        self.ctx = ctx
        self.lock = asyncio.Lock()
        self.evStopWatcher = asyncio.Event()
        self._byPeerId = collections.OrderedDict()

    @property
    def byPeerId(self):
        return self._byPeerId

    @property
    def peersIds(self):
        return self.byPeerId.keys()

    @property
    def peersCount(self):
        return len(self.peersIds)

    @property
    def peersNicknames(self):
        return [peer.ident.username for pid, peer in self.byPeerId.items()]

    async def unregister(self, peerId):
        if peerId in self.byPeerId:
            del self.byPeerId[peerId]
        self.peerLogout.emit(peerId)
        self.changed.emit()

    @ipfsOp
    async def registerFromIdent(self, op, iMsg):
        # identMsg is a PeerIdentMessage

        if not iMsg.peer in self.byPeerId:
            now = int(time.time())
            avgPing = await op.waitFor(op.pingAvg(iMsg.peer, count=2), 5)

            with await self.lock:
                pCtx = PeerCtx(self.ctx, iMsg.peer, iMsg,
                    pingavg=avgPing if avgPing else 0,
                    pinglast=now if avgPing else 0
                )
                self._byPeerId[iMsg.peer] = pCtx
            self.peerAdded.emit(iMsg.peer)
        else:
            with await self.lock:
                self.peerModified.emit(iMsg.peer)
                self._byPeerId[iMsg.peer].ident = iMsg

        self.changed.emit()

    async def init(self):
        pass

    async def watch(self):
        while True:
            await asyncio.sleep(60)

            with await self.lock:
                for peerId, pCtx in self.byPeerId.items():
                    if pCtx.peerUnresponsive:
                        log.debug('{} unresponsive ..'.format(peerId))
                        await self.unregister(peerId)

    async def stop(self):
        pass

    def getByPeerId(self, peerId):
        return self._byPeerId.get(peerId, None)

    def __str__(self):
        return 'Galacteek peers registered: {0}'.format(self.peersCount)

class Node(QObject):
    def __init__(self, parent):
        super().__init__(parent)
        self.ctx = parent
        self._id = None
        self._idFull = None

    @property
    def id(self):
        return self._id

    @property
    def idAll(self):
        return self._idFull

    @ipfsOp
    async def init(self, op):
        self._idFull = await op.client.core.id()
        self._id = await op.nodeId()

class PubsubMaster(QObject):
    psMessageRx = pyqtSignal()
    psMessageTx = pyqtSignal()

    def __init__(self, parent):
        super().__init__(parent)
        self.ctx = parent
        self._services = {}

    @property
    def services(self):
        return self._services

    def reg(self, service):
        self._services[service.topic] = service

    def status(self):
        [service.logStatus() for topic, service in self.services.items()]

    async def stop(self):
        tsks = [service.stop() for topic, service in self.services.items()]
        return await asyncio.gather(*tsks)

    def startServices(self):
        [service.start() for topic, service in self.services.items()]

    @ipfsOp
    async def init(self, op):
        pass

class P2PServices(QObject):
    def __init__(self, parent):
        super().__init__(parent)
        self.ctx = parent
        self._services = []

    @property
    def services(self):
        return self._services

    def servicesFormatted(self):
        return [ {
            'name': srv.name,
            'descr': srv.description,
            'protocol': srv.protocolName
        } for srv in self.services ]

    @ipfsOp
    async def init(self, op):
        if not await op.hasCommand('p2p') is True:
            log.debug('No P2P streams support')
            return

        log.debug('P2P streams support available')

    async def stop(self):
        for srv in self.services:
            await srv.stop()

class IPFSContext(QObject):
    # signals
    ipfsConnectionReady = pyqtSignal()
    ipfsRepositoryReady = pyqtSignal()

    # profiles
    profilesAvailable = pyqtSignal(list)
    profileChanged = pyqtSignal(str, UserProfile)
    profileInfoAvailable = pyqtSignal()

    # log events
    logAddProvider = pyqtSignal(dict)

    # pubsub
    pubsubMessageRx = pyqtSignal()
    pubsubMessageTx = pyqtSignal()

    pubsubMarksReceived = pyqtSignal(int)

    # pinning signals
    pinQueueSizeChanged = pyqtSignal(int)
    pinItemStatusChanged = pyqtSignal(str, dict)
    pinItemsCount = pyqtSignal(int)
    pinNewItem = pyqtSignal(str)
    pinFinished = pyqtSignal(str)

    def __init__(self, app):
        super().__init__()

        self._app = app

        self.objectStats = {}
        self.resources = {}
        self.profiles = {}
        self._currentProfile = None
        self.ipfsClient = None
        self._softIdent = None

        self.peers = Peers(self)
        self.node = Node(self)
        self.p2p = P2PServices(self)
        self.pubsub = PubsubMaster(self)

        self.pinner = None
        self.pinnerTask = None

    @property
    def app(self):
        return self._app

    @property
    def loop(self):
        return self.app.loop

    @property
    def client(self):
        return self.ipfsClient

    @property
    def currentProfile(self):
        return self._currentProfile

    @currentProfile.setter
    def currentProfile(self, p):
        log.debug('Switched profile: {}'.format(p.name))
        self._currentProfile = p

    @property
    def softIdent(self):
        return self._softIdent

    @softIdent.setter
    def softIdent(self, ident):
        log.debug('Software ident changed: CID {}'.format(ident['Hash']))
        self._softIdent = ident

    def hasRsc(self, name):
        return name in self.resources

    @ipfsOp
    async def setup(self, ipfsop, pubsubEnable=True,
            pubsubHashmarksExch=False, p2pEnable=False):
        self.rsaExec = RSAExecutor(loop=self.loop,
                executor=self.app.executor)
        await self.importSoftIdent()

        await self.node.init()
        await self.peers.init()

        if p2pEnable is True:
            await self.p2p.init()

        ensure(self.peers.watch())

        self.pinner = pinning.Pinner(self)
        self.pinnerTask = self.loop.create_task(self.pinner.process())

        if pubsubEnable is True:
            self.setupPubsub(pubsubHashmarksExch=pubsubHashmarksExch)

    async def shutdown(self):
        if self.pinnerTask:
            self.pinnerTask.cancel()
            await self.pinnerTask

        await self.peers.stop()
        await self.p2p.stop()
        await self.pubsub.stop()

    def setupPubsub(self, pubsubHashmarksExch=False):
        psServiceMain = PSMainService(self, self.app.ipfsClient)
        self.pubsub.reg(psServiceMain)

        psServicePeers = PSPeersService(self, self.app.ipfsClient)
        self.pubsub.reg(psServicePeers)

        if pubsubHashmarksExch:
            psServiceMarks = PSHashmarksExchanger(self, self.app.ipfsClient,
                 self.app.marksLocal, self.app.marksNetwork)
            self.pubsub.reg(psServiceMarks)

        self.pubsub.startServices()
        self.pubsub.status()

    @ipfsOp
    async def profilesInit(self, ipfsop):
        hasGalacteek = await ipfsop.filesLookup('/', GALACTEEK_NAME)
        if not hasGalacteek:
            await ipfsop.client.files.mkdir(GFILES_ROOT_PATH, parents=True)

        rootList = await ipfsop.filesList(GFILES_ROOT_PATH)

        # Scans existing profiles
        for entry in rootList:
            name = entry['Name']
            if entry['Type'] == 1 and name.startswith('profile.'):
                ma = re.search('profile\.([a-zA-Z\.\_\-]*)$', name)
                if ma:
                    profileName = ma.group(1).rstrip()
                    profile = await self.profileNew(profileName)

        defaultProfile = 'default'

        # Create default profile if not found
        if defaultProfile not in self.profiles:
            profile = await self.profileNew(defaultProfile)

        self.profileEmitAvail()
        self.profileChange(defaultProfile)

    def profileGet(self, name):
        return self.profiles.get(name, None)

    def profileEmitAvail(self):
        self.loop.call_soon(self.profilesAvailable.emit,
                list(self.profiles.keys()))

    async def profileNew(self, pName, emitavail=False):
        profile = UserProfile(self, pName,
                os.path.join(GFILES_ROOT_PATH, 'profile.{}'.format(pName)))
        try:
            await profile.init()
        except Exception as e:
            log.debug('Could not initialize profile: {}'.format(
                str(e)))
            return None
        self.profiles[pName] = profile
        if emitavail is True:
            self.profileEmitAvail()
        return profile

    def profileChange(self, pName):
        if pName in self.profiles:
            self.currentProfile = self.profiles[pName]
            self.profileChanged.emit(pName, self.currentProfile)
            return True
        else:
            return False

    @ipfsOp
    async def importSoftIdent(self, op):
        added = await op.client.add_str(gSoftIdent)
        if added is not None:
            self.softIdent = added

    @ipfsOp
    async def galacteekPeers(self, op):
        if self.softIdent is not None:
            return await op.whoProvides(self.softIdent['Hash'])
        else:
            return []



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/ipfssearchw.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_IPFSSearchMain(object):
    def setupUi(self, IPFSSearchMain):
        IPFSSearchMain.setObjectName("IPFSSearchMain")
        IPFSSearchMain.resize(576, 401)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(IPFSSearchMain)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.comboPages = QtWidgets.QComboBox(IPFSSearchMain)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboPages.sizePolicy().hasHeightForWidth())
        self.comboPages.setSizePolicy(sizePolicy)
        self.comboPages.setMinimumSize(QtCore.QSize(140, 0))
        self.comboPages.setObjectName("comboPages")
        self.horizontalLayout.addWidget(self.comboPages)
        self.prevPageButton = QtWidgets.QPushButton(IPFSSearchMain)
        self.prevPageButton.setText("")
        self.prevPageButton.setObjectName("prevPageButton")
        self.horizontalLayout.addWidget(self.prevPageButton)
        self.nextPageButton = QtWidgets.QPushButton(IPFSSearchMain)
        self.nextPageButton.setText("")
        self.nextPageButton.setObjectName("nextPageButton")
        self.horizontalLayout.addWidget(self.nextPageButton)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.labelInfo = QtWidgets.QLabel(IPFSSearchMain)
        self.labelInfo.setText("")
        self.labelInfo.setObjectName("labelInfo")
        self.horizontalLayout.addWidget(self.labelInfo)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(IPFSSearchMain)
        QtCore.QMetaObject.connectSlotsByName(IPFSSearchMain)

    def retranslateUi(self, IPFSSearchMain):
        _translate = QtCore.QCoreApplication.translate
        IPFSSearchMain.setWindowTitle(_translate("IPFSSearchMain", "Form"))





from PyQt5.QtWidgets import QDialog, QPushButton, QVBoxLayout, QAction
from PyQt5.QtWidgets import QTreeView, QHeaderView

from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtCore import Qt, QCoreApplication

from galacteek.ipfs.wrappers import ipfsOp

from . import ui_keys, ui_addkeydialog
from .modelhelpers import *
from .widgets import GalacteekTab
from .helpers import *
from .i18n import *

def iKeyName():
    return QCoreApplication.translate('KeysForm', 'Name')
def iKeyHash():
    return QCoreApplication.translate('KeysForm', 'Hash')
def iKeyResolve():
    return QCoreApplication.translate('KeysForm', 'Resolves to')

class AddKeyDialog(QDialog):
    def __init__(self, app, parent=None):
        super().__init__(parent)

        self.keysView = parent

        self.app = app
        self.ui = ui_addkeydialog.Ui_AddKeyDialog()
        self.ui.setupUi(self)

    def accept(self):
        keyName = self.ui.keyName.text()
        keySizeText = self.ui.keySize.currentText()

        self.app.task(self.createKey, keyName, int(keySizeText))

    @ipfsOp
    async def createKey(self, ipfsop, keyName, keySize):
        reply = await ipfsop.client.key.gen(keyName,
            type='rsa', size=keySize)
        self.done(1)
        self.keysView.updateKeysList()

    def reject(self):
        self.done(0)

class KeysView(QTreeView):
    def mousePressEvent (self, event):
        if event.button() == Qt.RightButton:
            pass
        QTreeView.mousePressEvent(self, event)

class KeysTab(GalacteekTab):
    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)

        self.ui = ui_keys.Ui_KeysForm()
        self.ui.setupUi(self)

        self.ui.addKeyButton.clicked.connect(self.onAddKeyClicked)
        self.ui.deleteKeyButton.clicked.connect(self.onDelKeyClicked)

        self.model = QStandardItemModel(parent=self)

        self.ui.treeKeys = KeysView()
        self.ui.treeKeys.doubleClicked.connect(self.onItemDoubleClicked)
        self.ui.treeKeys.setModel(self.model)

        self.ui.verticalLayout.addWidget(self.ui.treeKeys)

        self.setupModel()
        self.app.task(self.listKeys)

    def setupModel(self):
        self.model.clear()
        self.model.setColumnCount(3)
        self.model.setHorizontalHeaderLabels([
            iKeyName(), iKeyHash(), iKeyResolve()])
        self.ui.treeKeys.header().setSectionResizeMode(0,
            QHeaderView.ResizeToContents)
        self.ui.treeKeys.header().setSectionResizeMode(1,
            QHeaderView.ResizeToContents)

    def onDelKeyClicked(self):
        idx = self.ui.treeKeys.currentIndex()
        idxName = self.model.index(idx.row(), 0, idx.parent())
        keyName = self.model.data(idxName)
        if keyName:
            self.app.task(self.delKey, keyName)

    def onAddKeyClicked(self):
        runDialog(AddKeyDialog, self.app, parent=self)

    @ipfsOp
    async def delKey(self, ipfsop, name):
        if await ipfsop.keysRemove(name):
            modelDelete(self.model, name)
        self.updateKeysList()

    @ipfsOp
    async def listKeys(self, ipfsop):
        keys = await ipfsop.keys()
        for key in keys:
            found = modelSearch(self.model, search=key['Name'])
            if len(found) > 0:
                continue
            resolveItem = UneditableItem('')
            self.model.appendRow([
                UneditableItem(key['Name']),
                UneditableItem(key['Id']),
                resolveItem
            ])
            self.app.task(self.keyResolve, key, resolveItem)

    @ipfsOp
    async def keyResolve(self, ipfsop, key, item):
        resolved = await ipfsop.resolve(key['Id'])
        if resolved:
            item.setText(resolved['Path'])
        else:
            item.setText(iUnknown())

    def updateKeysList(self):
        self.app.task(self.listKeys)

    def onItemDoubleClicked(self, index):
        # Browse IPNS key associated with current item on double-click
        keyHash = self.model.data(self.model.index(index.row(), 1))
        self.gWindow.addBrowserTab().browseIpnsHash(keyHash)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/peersmgr.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_PeersManager(object):
    def setupUi(self, PeersManager):
        PeersManager.setObjectName("PeersManager")
        PeersManager.resize(639, 422)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(PeersManager)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label = QtWidgets.QLabel(PeersManager)
        self.label.setObjectName("label")
        self.horizontalLayout_3.addWidget(self.label)
        self.peersCountLabel = QtWidgets.QLabel(PeersManager)
        self.peersCountLabel.setText("")
        self.peersCountLabel.setObjectName("peersCountLabel")
        self.horizontalLayout_3.addWidget(self.peersCountLabel)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.label_2 = QtWidgets.QLabel(PeersManager)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.search = QtWidgets.QLineEdit(PeersManager)
        self.search.setMaximumSize(QtCore.QSize(250, 16777215))
        self.search.setMaxLength(128)
        self.search.setObjectName("search")
        self.horizontalLayout_3.addWidget(self.search)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.tree = QtWidgets.QTreeView(PeersManager)
        self.tree.setObjectName("tree")
        self.verticalLayout.addWidget(self.tree)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(PeersManager)
        QtCore.QMetaObject.connectSlotsByName(PeersManager)

    def retranslateUi(self, PeersManager):
        _translate = QtCore.QCoreApplication.translate
        PeersManager.setWindowTitle(_translate("PeersManager", "Form"))
        self.label.setText(_translate("PeersManager", "Peers registered"))
        self.label_2.setText(_translate("PeersManager", "Search"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/donatedialog.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DonateDialog(object):
    def setupUi(self, DonateDialog):
        DonateDialog.setObjectName("DonateDialog")
        DonateDialog.resize(538, 286)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(DonateDialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.textBrowser = QtWidgets.QTextBrowser(DonateDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textBrowser.sizePolicy().hasHeightForWidth())
        self.textBrowser.setSizePolicy(sizePolicy)
        self.textBrowser.setMaximumSize(QtCore.QSize(16777215, 200))
        self.textBrowser.setObjectName("textBrowser")
        self.verticalLayout.addWidget(self.textBrowser)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.label_3 = QtWidgets.QLabel(DonateDialog)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_2.addWidget(self.label_3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.bitcoinAddress = QtWidgets.QLabel(DonateDialog)
        self.bitcoinAddress.setText("")
        self.bitcoinAddress.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.bitcoinAddress.setObjectName("bitcoinAddress")
        self.horizontalLayout_2.addWidget(self.bitcoinAddress)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.bitcoinClip = QtWidgets.QPushButton(DonateDialog)
        self.bitcoinClip.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.bitcoinClip.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/share/icons/clipboard.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bitcoinClip.setIcon(icon)
        self.bitcoinClip.setObjectName("bitcoinClip")
        self.horizontalLayout_2.addWidget(self.bitcoinClip)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.okButton = QtWidgets.QPushButton(DonateDialog)
        self.okButton.setMinimumSize(QtCore.QSize(200, 0))
        self.okButton.setObjectName("okButton")
        self.verticalLayout_2.addWidget(self.okButton, 0, QtCore.Qt.AlignRight)

        self.retranslateUi(DonateDialog)
        QtCore.QMetaObject.connectSlotsByName(DonateDialog)

    def retranslateUi(self, DonateDialog):
        _translate = QtCore.QCoreApplication.translate
        DonateDialog.setWindowTitle(_translate("DonateDialog", "Dialog"))
        self.textBrowser.setHtml(_translate("DonateDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">Any donation is welcome and will help support other projects like </span><a href=\"https://ipfs-search.com\"><span style=\" text-decoration: underline; color:#0000ff;\">IPFS Search</span></a></p>\n"
"<p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">You can make a donation with Bitcoin using the address below.</span></p>\n"
"<p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">Thanks!</span></p></body></html>"))
        self.label_3.setText(_translate("DonateDialog", "Bitcoin donation address"))
        self.okButton.setText(_translate("DonateDialog", "OK"))

from . import galacteek_rc



import sys

from logbook import Logger, StreamHandler
from logbook.compat import redirect_logging
from logbook.helpers import u

mainFormatString = u(
    '[{record.time:%Y-%m-%d %H:%M:%S.%f%z}] '
    '{record.level_name}: {record.channel}/{record.module}: {record.message}')

def basicConfig(level='INFO'):
    handler = StreamHandler(sys.stderr, level=level)
    handler.format_string = mainFormatString
    handler.push_application()

loggerMain = Logger('galacteek')



import asyncio
import aiohttp

from distutils.version import StrictVersion

from galacteek import __version__

async def getLatestVersion(pkgname='galacteek'):
    url = 'https://pypi.org/pypi/{0}/json'.format(pkgname)

    async with aiohttp.ClientSession() as session:
        async with session.get(url) as resp:
            data = await resp.json()
            return StrictVersion(data['info']['version'])

async def newReleaseAvailable():
    try:
        latestV = await getLatestVersion()
        currentV = StrictVersion(__version__)
        return latestV > currentV
    except asyncio.TimeoutError:
        return False
    except:
        return False



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/ipfsmultiplecidinputdialog.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MultipleCIDInputDialog(object):
    def setupUi(self, MultipleCIDInputDialog):
        MultipleCIDInputDialog.setObjectName("MultipleCIDInputDialog")
        MultipleCIDInputDialog.resize(745, 200)
        self.verticalLayout = QtWidgets.QVBoxLayout(MultipleCIDInputDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(MultipleCIDInputDialog)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.cid = QtWidgets.QLineEdit(MultipleCIDInputDialog)
        self.cid.setObjectName("cid")
        self.horizontalLayout.addWidget(self.cid)
        self.addCidButton = QtWidgets.QPushButton(MultipleCIDInputDialog)
        self.addCidButton.setMaximumSize(QtCore.QSize(40, 16777215))
        self.addCidButton.setObjectName("addCidButton")
        self.horizontalLayout.addWidget(self.addCidButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.validity = QtWidgets.QLabel(MultipleCIDInputDialog)
        self.validity.setText("")
        self.validity.setAlignment(QtCore.Qt.AlignCenter)
        self.validity.setObjectName("validity")
        self.horizontalLayout_3.addWidget(self.validity)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.cidList = QtWidgets.QListWidget(MultipleCIDInputDialog)
        self.cidList.setObjectName("cidList")
        self.horizontalLayout_2.addWidget(self.cidList)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.verticalLayout.addLayout(self.verticalLayout_2)
        self.buttonBox = QtWidgets.QDialogButtonBox(MultipleCIDInputDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(MultipleCIDInputDialog)
        self.buttonBox.accepted.connect(MultipleCIDInputDialog.accept)
        self.buttonBox.rejected.connect(MultipleCIDInputDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(MultipleCIDInputDialog)

    def retranslateUi(self, MultipleCIDInputDialog):
        _translate = QtCore.QCoreApplication.translate
        MultipleCIDInputDialog.setWindowTitle(_translate("MultipleCIDInputDialog", "Dialog"))
        self.label.setText(_translate("MultipleCIDInputDialog", "IPFS CID"))
        self.addCidButton.setToolTip(_translate("MultipleCIDInputDialog", "<html><head/><body><p>Add CID to the list</p></body></html>"))
        self.addCidButton.setText(_translate("MultipleCIDInputDialog", "+"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/dagview.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DagViewForm(object):
    def setupUi(self, DagViewForm):
        DagViewForm.setObjectName("DagViewForm")
        DagViewForm.resize(400, 300)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(DagViewForm)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.dagHash = QtWidgets.QLabel(DagViewForm)
        self.dagHash.setText("")
        self.dagHash.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.dagHash.setObjectName("dagHash")
        self.verticalLayout.addWidget(self.dagHash)
        self.dagTree = QtWidgets.QTreeWidget(DagViewForm)
        self.dagTree.setColumnCount(2)
        self.dagTree.setObjectName("dagTree")
        self.dagTree.headerItem().setText(0, "1")
        self.dagTree.headerItem().setText(1, "2")
        self.verticalLayout.addWidget(self.dagTree)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(DagViewForm)
        QtCore.QMetaObject.connectSlotsByName(DagViewForm)

    def retranslateUi(self, DagViewForm):
        _translate = QtCore.QCoreApplication.translate
        DagViewForm.setWindowTitle(_translate("DagViewForm", "Form"))





import asyncio
import time

from PyQt5.QtCore import (pyqtSignal, QObject)

from galacteek import log, ensure, asyncify
from galacteek.ipfs.wrappers import ipfsOp
from galacteek.core.jtraverse import traverseParser

class MutableIPFSJson(QObject):
    """
    Mutable IPFS JSON object.

    As you make changes to the JSON object contained within this object, a copy
    of that JSON is maintained in the MFS at the given path

    Signals:
        changed: emit this signal when you changed the object and it will
            trigger saving
        entryChanged: emitted when the object has been flushed to IPFS and is
            represented by a new hash
        available: object has been loaded (emitted once on init)

    :param str mfsFilePath: the path inside the MFS for this object
    """

    changed = pyqtSignal()
    entryChanged = pyqtSignal()
    available = pyqtSignal(dict)

    def __init__(self, mfsFilePath,  **kw):
        super().__init__()

        self.lock = asyncio.Lock()
        self.loaded = asyncio.Future()
        self.evLoaded = asyncio.Event()
        self._root = None
        self._avail = False
        self._curEntry = None
        self._entryHistory = []
        self._mfsFilePath = mfsFilePath
        self.__dict__.update(kw)

        self.changed.connect(self.onChanged)
        self.available.connect(self.onObjAvailable)

    def debug(self, msg):
        log.debug('MFS JSON {0}: {1}'.format(self.mfsFilePath, msg))

    def onObjAvailable(self, r):
        self._avail = True

    def onChanged(self):
        self.save()

    def initObj(self):
        raise Exception('implement initObj')

    @property
    def mfsFilePath(self):
        return self._mfsFilePath # path inside the mfs

    @property
    def curEntry(self):
        """
        The current IPFS entry representing this object
        """
        return self._curEntry

    @property
    def avail(self):
        return self._avail

    @curEntry.setter
    def curEntry(self, value):
        self._entryHistory.append((time.time(), value))
        self._curEntry = value
        self.entryChanged.emit()

    @property
    def root(self):
        return self._root

    async def load(self):
        await self.loadIpfsObj()

    @ipfsOp
    async def loadIpfsObj(self, op):
        """
        Load the JSON from the MFS if it already exists or initialize a new
        object otherwise
        """
        self.debug('Loading mutable object from {}'.format(self.mfsFilePath))

        obj = await op.filesReadJsonObject(self.mfsFilePath)

        if obj:
            self.debug('Successfully loaded JSON')
            self._root = obj
            self.curEntry = await op.filesStat(self.mfsFilePath)
            self.loaded.set_result(True)
            self.evLoaded.set()
        else:
            self.debug('JSON empty or invalid, initializing')
            self._root = self.initObj()
            await self.ipfsSave()
            self.loaded.set_result(True)
            self.evLoaded.set()

        self.parser = traverseParser(self.root)
        if self.upgrade() is True:
            await self.ipfsSave()

        self.available.emit(self.root)

    @asyncify
    async def save(self):
        await self.ipfsSave()

    @ipfsOp
    async def ipfsSave(self, op):
        with await self.lock:
            resp = await op.filesWriteJsonObject(self.mfsFilePath, self.root)

            if resp is not None:
                self.curEntry = await op.filesStat(self.mfsFilePath)
                return True
            else:
                return False

    def upgrade(self):
        return False



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/keys.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_KeysForm(object):
    def setupUi(self, KeysForm):
        KeysForm.setObjectName("KeysForm")
        KeysForm.resize(606, 407)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(KeysForm)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.addKeyButton = QtWidgets.QPushButton(KeysForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.addKeyButton.sizePolicy().hasHeightForWidth())
        self.addKeyButton.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/share/icons/key-diago.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.addKeyButton.setIcon(icon)
        self.addKeyButton.setIconSize(QtCore.QSize(16, 16))
        self.addKeyButton.setObjectName("addKeyButton")
        self.horizontalLayout.addWidget(self.addKeyButton, 0, QtCore.Qt.AlignLeft)
        self.deleteKeyButton = QtWidgets.QPushButton(KeysForm)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.deleteKeyButton.sizePolicy().hasHeightForWidth())
        self.deleteKeyButton.setSizePolicy(sizePolicy)
        self.deleteKeyButton.setObjectName("deleteKeyButton")
        self.horizontalLayout.addWidget(self.deleteKeyButton, 0, QtCore.Qt.AlignLeft)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_3.addLayout(self.verticalLayout)

        self.retranslateUi(KeysForm)
        QtCore.QMetaObject.connectSlotsByName(KeysForm)

    def retranslateUi(self, KeysForm):
        _translate = QtCore.QCoreApplication.translate
        KeysForm.setWindowTitle(_translate("KeysForm", "Form"))
        self.addKeyButton.setText(_translate("KeysForm", "Add key"))
        self.deleteKeyButton.setText(_translate("KeysForm", "Delete"))

from . import galacteek_rc




import asyncio

import aioipfs

from galacteek import log
from galacteek.ipfs.wrappers import ipfsOp

class Pinner(object):
    """ Pins objects on request through an async queue """

    def __init__(self, ctx):
        self._pinQueue = asyncio.Queue()
        self._ctx = ctx
        self.lock = asyncio.Lock()
        self._pinStatus = {}

    @property
    def pinQueue(self):
        return self._pinQueue

    @property
    def ipfsCtx(self):
        return self._ctx

    @property
    def status(self):
        return self._pinStatus

    def debug(self, msg):
        log.debug('Pinning service: {}'.format(msg))

    def _emitSLength(self):
        self.ipfsCtx.pinItemsCount.emit(len(self._pinStatus))

    def pathRegister(self, path):
        self._pinStatus[path] = {}
        self.ipfsCtx.pinNewItem.emit(path)
        self._emitSLength()

    def pathDelete(self, path):
        if path in self._pinStatus:
            del self._pinStatus[path]
        self._emitSLength()

    @ipfsOp
    async def pin(self, op, path, recursive=False):
        self.debug('pinning object {0}, recursive is {1}'.format(
            path, recursive))
        self.pathRegister(path)

        try:
            async for pinned in op.client.pin.add(path, recursive=recursive):
                self.debug('Pinning progress {0}: {1}'.format(path, pinned))
                await asyncio.sleep(0)
                with await self.lock:
                    self._pinStatus[path] = pinned
                self.ipfsCtx.pinItemStatusChanged.emit(path, pinned)
        except aioipfs.APIError as err:
            self.debug('Pinning error {0}: {1}'.format(path, str(err.message)))

        with await self.lock:
            self.pathDelete(path)

        self.ipfsCtx.pinFinished.emit(path)
        return path

    async def queue(self, path, recursive, onSuccess):
        """ Queue an item for processing """
        await self.pinQueue.put((path, recursive, onSuccess))
        self.ipfsCtx.pinQueueSizeChanged.emit(self.pinQueue.qsize())

    async def process(self):
        try:
            while True:
                item = await self.pinQueue.get()
                if item is None:
                    self.debug('null item in queue')
                    continue

                self.ipfsCtx.pinQueueSizeChanged.emit(self.pinQueue.qsize())
                path, recursive, callback = item
                f = asyncio.ensure_future(self.pin(path, recursive=recursive))
                if callback:
                    f.add_done_callback(callback)
        except asyncio.CancelledError as err:
            self.debug('cancelled error: {}'.format(str(err)))



import pytest

from galacteek.appsettings import *

@pytest.fixture
def mgr(tmpdir):
    filep = tmpdir.join('settings.txt')
    return SettingsManager(str(filep))

class TestSettings:
    @pytest.mark.parametrize('sec', ['SEC1', 'SEC190'])
    @pytest.mark.parametrize('name', ['NAME1'])
    @pytest.mark.parametrize('value', ['POID1233902'])
    def test_simple(self, mgr, sec, name, value):
        mgr.setDefaultSetting(sec, name, value)

        assert mgr.getSetting(sec, name) == value
        with pytest.raises(Exception):
            mgr.getSetting(sec, name, int)

    @pytest.mark.parametrize('sec', ['SEC1'])
    @pytest.mark.parametrize('name', ['NAME1'])
    def test_booleans(self, mgr, sec, name):
        mgr.setTrue(sec, name)

        assert mgr.isTrue(sec, name) == True
        assert mgr.isFalse(sec, name) == False



# Galacteek, GUI startup entry point

import asyncio
import sys
import argparse
import shutil
import os, os.path
import subprocess
import logging
import importlib
from distutils.version import StrictVersion

from PyQt5.QtWidgets import QApplication

from galacteek import log, ensure
from galacteek.core import glogger

from galacteek.ipfs import distipfsfetch
from galacteek.ui import mainui
from galacteek.ui.helpers import *
from galacteek.ui.i18n import *
from galacteek import application
from galacteek.appsettings import *

try:
    import aiomonitor
except ImportError:
    haveAiomonitor = False
else:
    haveAiomonitor = True

def whichIpfs():
    return shutil.which('ipfs')

def ipfsVersion():
    try:
        p = subprocess.Popen(['ipfs', 'version', '-n'], stdout=subprocess.PIPE)
        out, err = p.communicate()
        return StrictVersion(out.decode().strip())
    except:
        return None

async def fetchGoIpfsWrapper(app, timeout=60*10):
    try:
        await asyncio.wait_for(fetchGoIpfsDist(app), timeout)
    except asyncio.TimeoutError as e:
        app.mainWindow.statusMessage(iGoIpfsFetchTimeout())
        return None
    except Exception as e:
        app.mainWindow.statusMessage(iGoIpfsFetchError())
        return None
    else:
        app.mainWindow.statusMessage(iGoIpfsFetchSuccess())
        return whichIpfs()

async def fetchGoIpfsDist(app):
    async for msg in distipfsfetch.distIpfsExtract(
            dstdir=app.ipfsBinLocation, software='go-ipfs',
            executable='ipfs', version='0.4.17', loop=app.loop):
        try:
            code, text = msg
            app.mainWindow.statusMessage(text)
        except Exception as e:
            app.debug(str(e))

def galacteekGui(args):
    if args.debug:
        glogger.basicConfig(level='DEBUG')
    else:
        glogger.basicConfig(level='INFO')

    gApp = application.GalacteekApplication(profile=args.profile,
            debug=args.debug)
    loop = gApp.setupAsyncLoop()
    sManager = gApp.settingsMgr

    section = CFG_SECTION_IPFSD
    if args.apiport:
        sManager.setSetting(section, CFG_KEY_APIPORT, args.apiport)
    if args.swarmport:
        sManager.setSetting(section, CFG_KEY_SWARMPORT, args.swarmport)
    if args.gatewayport:
        sManager.setSetting(section, CFG_KEY_HTTPGWPORT, args.gatewayport)

    if sManager.isTrue(CFG_SECTION_IPFSD, CFG_KEY_ENABLED):
        fsMigratePath = shutil.which('fs-repo-migrations')
        hasFsMigrate = fsMigratePath is not None

        if hasFsMigrate is False and args.migrate is True:
            gApp.systemTrayMessage('Galacteek', iFsRepoMigrateNotFound())

        enableMigrate = hasFsMigrate is True and args.migrate is True

        # Look if we can find the ipfs executable
        ipfsPath = whichIpfs()
        if not ipfsPath:
            fetchWanted = questionBox('go-ipfs', iGoIpfsFetchAsk())

            def fetchFinished(fut):
                path = fut.result()
                if path is None:
                    gApp.systemTrayMessage('Galacteek',
                            iGoIpfsFetchError())
                else:
                    gApp.startIpfsDaemon(goIpfsPath=path,
                            migrateRepo=enableMigrate)

            if fetchWanted:
                fut = ensure(fetchGoIpfsWrapper(gApp))
                fut.add_done_callback(fetchFinished)
            else:
                gApp.systemTrayMessage('Galacteek',
                        iGoIpfsNotFound())
        else:
            minVersion = StrictVersion('0.4.7')
            version = ipfsVersion()

            if version < minVersion:
                # warning here
                log.debug('go-ipfs version found {0} is too old'.format(
                    version))
                gApp.systemTrayMessage('Galacteek',
                        iGoIpfsTooOld())

            gApp.startIpfsDaemon(goIpfsPath=ipfsPath,
                    migrateRepo=enableMigrate)
    else:
        ensure(gApp.updateIpfsClient())

    gApp.mainWindow.addHashmarksTab()

    if args.noreleasecheck is False:
        ensure(gApp.checkReleases())

    # Monitor event loop if aiomonitor is available
    # Use the context manager so loop cleanup/close is automatic

    if args.monitor is True and haveAiomonitor:
        with aiomonitor.start_monitor(loop=loop):
            with loop:
                loop.run_forever()
    else:
        with loop:
            loop.run_forever()

def start():
    parser = argparse.ArgumentParser()

    parser.add_argument('--apiport',  default=None,
        help='IPFS API port number')
    parser.add_argument('--swarmport', default=None,
        help='IPFS swarm port number')
    parser.add_argument('--gatewayport', default=None,
        help='IPFS http gateway port number')
    parser.add_argument('--profile', default='main',
        help='Application Profile')
    parser.add_argument('--monitor', action='store_true',
        dest='monitor', help = 'Monitor application with aiomonitor')
    parser.add_argument('--migrate', action='store_true',
        dest='migrate', help = 'Activate automatic repository migration')
    parser.add_argument('--no-release-check', action='store_true',
        dest='noreleasecheck', help = "Don't check for new releases on PyPI")
    parser.add_argument('-d', action='store_true',
        dest='debug', help = 'Activate debugging')
    args = parser.parse_args()

    galacteekGui(args)




import json
import asyncio

from PyQt5.QtWidgets import QTreeWidget, QTreeWidgetItem
from PyQt5.QtCore import Qt, QCoreApplication
from PyQt5.QtGui import QBrush, QColor

from galacteek.appsettings import *
from galacteek.ipfs import cidhelpers
from galacteek.ipfs.ipfsops import *
from galacteek.ipfs.wrappers import ipfsOp, ipfsStatOp
from .helpers import *
from .i18n import *
from .widgets import GalacteekTab

from . import ui_dagview

def iDagError(path):
    return QCoreApplication.translate('DagViewForm',
            'Error loading the DAG object: {0}').format(path)

def iDagInfo(obj):
    return QCoreApplication.translate('DagViewForm',
            'DAG object: <b>{0}</b>').format(obj)

def iDagItem(it):
    return QCoreApplication.translate('DagViewForm',
            'Item {0}').format(it)

class DAGViewer(GalacteekTab):
    def __init__(self, dagHash, *args, **kw):
        super(DAGViewer, self).__init__(*args, **kw)

        self.dagHash = dagHash

        self.ui = ui_dagview.Ui_DagViewForm()
        self.ui.setupUi(self)

        evfilter = IPFSTreeKeyFilter(self.ui.dagTree)
        evfilter.copyHashPressed.connect(self.onCopyItemHash)
        self.ui.dagTree.installEventFilter(evfilter)

        self.ui.dagTree.itemDoubleClicked.connect(self.onItemDoubleClicked)
        self.ui.dagTree.setHeaderLabels([iKey(), iValue()])
        self.ui.dagHash.setText(iDagInfo(self.dagHash))

        self.rootItem = self.ui.dagTree.invisibleRootItem()
        self.app.task(self.loadDag, self.dagHash, self.rootItem)

    def onCopyItemHash(self):
        currentItem = self.ui.dagTree.currentItem()
        value = currentItem.text(1)
        if cidhelpers.cidValid(value):
            self.app.setClipboardText(value)

    def onItemDoubleClicked(self, item, col):
        if col == 1:
            text = item.text(col)
            if text.startswith('/ipfs/'):
                self.gWindow.addBrowserTab().browseFsPath(text)
            elif cidhelpers.cidValid(text):
                self.gWindow.addBrowserTab().browseIpfsHash(text)

    @ipfsOp
    async def loadDag(self, ipfsop, hashRef, item=None):
        if item is None:
            item = self.rootItem
        await asyncio.sleep(0)
        dagNode = await ipfsop.dagGet(hashRef)
        if not dagNode:
            return messageBox(iDagError(hashRef))

        await self.displayItem(dagNode, item)
        item.setExpanded(True)
        self.ui.dagTree.resizeColumnToContents(0)

    async def displayItem(self, data, item):
        if data is None:
            return
        await asyncio.sleep(0)
        if isinstance(data, dict):
            for dkey, dval in data.items():
                await asyncio.sleep(0)
                if dkey == '/':
                    # Merkle-link
                    linkValue = str(dval)
                    dictItem = QTreeWidgetItem(item)
                    dictItem.setText(0, iMerkleLink())
                    dictItem.setText(1, linkValue)
                    color = QColor(212, 184, 116)
                    dictItem.setBackground(0, color)
                    dictItem.setBackground(1, color)
                    dictItem.setExpanded(True)
                else:
                    dictItem = QTreeWidgetItem(item)
                    dictItem.setText(0, dkey)
                    dictItem.setExpanded(True)
                    if isinstance(dval, str):
                        dictItem.setText(1, dval)
                    elif isinstance(dval, int):
                        dictItem.setText(1, str(dval))
                    else:
                        await self.displayItem(dval, dictItem)

        elif isinstance(data, list):
            for i in range(0, len(data)):
                await asyncio.sleep(0)
                listItem = QTreeWidgetItem(item)
                listItem.setText(0, iDagItem(i))
                listItem.setExpanded(True)
                await self.displayItem(data[i], listItem)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/addhashmarkdialog.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_AddHashmarkDialog(object):
    def setupUi(self, AddHashmarkDialog):
        AddHashmarkDialog.setObjectName("AddHashmarkDialog")
        AddHashmarkDialog.resize(1071, 390)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(AddHashmarkDialog.sizePolicy().hasHeightForWidth())
        AddHashmarkDialog.setSizePolicy(sizePolicy)
        AddHashmarkDialog.setMaximumSize(QtCore.QSize(1071, 16777215))
        self.formLayout_2 = QtWidgets.QFormLayout(AddHashmarkDialog)
        self.formLayout_2.setObjectName("formLayout_2")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.resourceLabel = QtWidgets.QLabel(AddHashmarkDialog)
        self.resourceLabel.setMinimumSize(QtCore.QSize(800, 0))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.resourceLabel.setFont(font)
        self.resourceLabel.setText("")
        self.resourceLabel.setObjectName("resourceLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.resourceLabel)
        self.label = QtWidgets.QLabel(AddHashmarkDialog)
        self.label.setObjectName("label")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label)
        self.title = QtWidgets.QLineEdit(AddHashmarkDialog)
        self.title.setObjectName("title")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.title)
        self.label_2 = QtWidgets.QLabel(AddHashmarkDialog)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.category = QtWidgets.QComboBox(AddHashmarkDialog)
        self.category.setObjectName("category")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.category)
        self.label_4 = QtWidgets.QLabel(AddHashmarkDialog)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.newCategory = QtWidgets.QLineEdit(AddHashmarkDialog)
        self.newCategory.setMaxLength(128)
        self.newCategory.setObjectName("newCategory")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.newCategory)
        self.label_3 = QtWidgets.QLabel(AddHashmarkDialog)
        self.label_3.setText("")
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.share = QtWidgets.QCheckBox(AddHashmarkDialog)
        self.share.setObjectName("share")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.share)
        self.label_5 = QtWidgets.QLabel(AddHashmarkDialog)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.tags = QtWidgets.QLineEdit(AddHashmarkDialog)
        self.tags.setMaxLength(256)
        self.tags.setObjectName("tags")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.tags)
        self.label_6 = QtWidgets.QLabel(AddHashmarkDialog)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.label_6)
        self.comment = QtWidgets.QLineEdit(AddHashmarkDialog)
        self.comment.setObjectName("comment")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.comment)
        self.label_7 = QtWidgets.QLabel(AddHashmarkDialog)
        self.label_7.setObjectName("label_7")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_7)
        self.description = QtWidgets.QTextEdit(AddHashmarkDialog)
        self.description.setAcceptRichText(False)
        self.description.setObjectName("description")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.description)
        self.formLayout_2.setLayout(2, QtWidgets.QFormLayout.FieldRole, self.formLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(AddHashmarkDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.formLayout_2.setWidget(14, QtWidgets.QFormLayout.SpanningRole, self.buttonBox)

        self.retranslateUi(AddHashmarkDialog)
        self.buttonBox.accepted.connect(AddHashmarkDialog.accept)
        self.buttonBox.rejected.connect(AddHashmarkDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(AddHashmarkDialog)

    def retranslateUi(self, AddHashmarkDialog):
        _translate = QtCore.QCoreApplication.translate
        AddHashmarkDialog.setWindowTitle(_translate("AddHashmarkDialog", "Add hashmark dialog"))
        self.label.setText(_translate("AddHashmarkDialog", "Use title for hashmark"))
        self.label_2.setText(_translate("AddHashmarkDialog", "Category"))
        self.label_4.setText(_translate("AddHashmarkDialog", "or create new category, e.g news/world"))
        self.share.setText(_translate("AddHashmarkDialog", "Share (public)"))
        self.label_5.setText(_translate("AddHashmarkDialog", "Tags list"))
        self.label_6.setText(_translate("AddHashmarkDialog", "Comment"))
        self.label_7.setText(_translate("AddHashmarkDialog", "Description"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'galacteek/ui/profileeditdialog.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ProfileEditDialog(object):
    def setupUi(self, ProfileEditDialog):
        ProfileEditDialog.setObjectName("ProfileEditDialog")
        ProfileEditDialog.resize(621, 348)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(ProfileEditDialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(ProfileEditDialog)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_2 = QtWidgets.QLabel(self.tab)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 4, 0, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.tab)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 5, 0, 1, 1)
        self.firstname = QtWidgets.QLineEdit(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.firstname.sizePolicy().hasHeightForWidth())
        self.firstname.setSizePolicy(sizePolicy)
        self.firstname.setMaxLength(64)
        self.firstname.setObjectName("firstname")
        self.gridLayout_2.addWidget(self.firstname, 4, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.tab)
        self.label_4.setObjectName("label_4")
        self.gridLayout_2.addWidget(self.label_4, 6, 0, 1, 1)
        self.label = QtWidgets.QLabel(self.tab)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 2, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
        self.gridLayout_2.addItem(spacerItem, 9, 1, 1, 1)
        self.lastname = QtWidgets.QLineEdit(self.tab)
        self.lastname.setMaxLength(128)
        self.lastname.setObjectName("lastname")
        self.gridLayout_2.addWidget(self.lastname, 5, 1, 1, 1)
        self.email = QtWidgets.QLineEdit(self.tab)
        self.email.setMaxLength(128)
        self.email.setObjectName("email")
        self.gridLayout_2.addWidget(self.email, 6, 1, 1, 1)
        self.username = QtWidgets.QLineEdit(self.tab)
        self.username.setMaxLength(128)
        self.username.setObjectName("username")
        self.gridLayout_2.addWidget(self.username, 2, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.tab)
        self.label_5.setObjectName("label_5")
        self.gridLayout_2.addWidget(self.label_5, 11, 0, 1, 1)
        self.profileCryptoId = QtWidgets.QLabel(self.tab)
        self.profileCryptoId.setText("")
        self.profileCryptoId.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.profileCryptoId.setObjectName("profileCryptoId")
        self.gridLayout_2.addWidget(self.profileCryptoId, 11, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.tab)
        self.label_6.setObjectName("label_6")
        self.gridLayout_2.addWidget(self.label_6, 7, 0, 1, 1)
        self.org = QtWidgets.QLineEdit(self.tab)
        self.org.setMaxLength(128)
        self.org.setObjectName("org")
        self.gridLayout_2.addWidget(self.org, 7, 1, 1, 1)
        self.labelWarning = QtWidgets.QLabel(self.tab)
        self.labelWarning.setWordWrap(True)
        self.labelWarning.setObjectName("labelWarning")
        self.gridLayout_2.addWidget(self.labelWarning, 1, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Maximum)
        self.gridLayout_2.addItem(spacerItem1, 3, 1, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.tab)
        self.label_7.setObjectName("label_7")
        self.gridLayout_2.addWidget(self.label_7, 8, 0, 1, 1)
        self.countryBox = QtWidgets.QComboBox(self.tab)
        self.countryBox.setObjectName("countryBox")
        self.gridLayout_2.addWidget(self.countryBox, 8, 1, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.iconHash = QtWidgets.QLabel(self.tab_2)
        self.iconHash.setText("")
        self.iconHash.setObjectName("iconHash")
        self.verticalLayout_4.addWidget(self.iconHash)
        self.iconPixmap = QtWidgets.QLabel(self.tab_2)
        self.iconPixmap.setText("")
        self.iconPixmap.setObjectName("iconPixmap")
        self.verticalLayout_4.addWidget(self.iconPixmap)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem2)
        self.changeIconButton = QtWidgets.QPushButton(self.tab_2)
        self.changeIconButton.setObjectName("changeIconButton")
        self.verticalLayout_4.addWidget(self.changeIconButton)
        self.verticalLayout_3.addLayout(self.verticalLayout_4)
        self.tabWidget.addTab(self.tab_2, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.cancelButton = QtWidgets.QPushButton(ProfileEditDialog)
        self.cancelButton.setMaximumSize(QtCore.QSize(120, 16777215))
        self.cancelButton.setObjectName("cancelButton")
        self.horizontalLayout_2.addWidget(self.cancelButton)
        self.updateButton = QtWidgets.QPushButton(ProfileEditDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.updateButton.sizePolicy().hasHeightForWidth())
        self.updateButton.setSizePolicy(sizePolicy)
        self.updateButton.setMaximumSize(QtCore.QSize(120, 16777215))
        self.updateButton.setObjectName("updateButton")
        self.horizontalLayout_2.addWidget(self.updateButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.retranslateUi(ProfileEditDialog)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(ProfileEditDialog)

    def retranslateUi(self, ProfileEditDialog):
        _translate = QtCore.QCoreApplication.translate
        ProfileEditDialog.setWindowTitle(_translate("ProfileEditDialog", "Dialog"))
        self.label_2.setText(_translate("ProfileEditDialog", "First name"))
        self.label_3.setText(_translate("ProfileEditDialog", "Last name"))
        self.label_4.setText(_translate("ProfileEditDialog", "Email"))
        self.label.setText(_translate("ProfileEditDialog", "Username"))
        self.label_5.setText(_translate("ProfileEditDialog", "Current cryptographic identifier"))
        self.label_6.setText(_translate("ProfileEditDialog", "Organization"))
        self.labelWarning.setText(_translate("ProfileEditDialog", "Only the username is strictly required"))
        self.label_7.setText(_translate("ProfileEditDialog", "Country"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("ProfileEditDialog", "User information"))
        self.changeIconButton.setText(_translate("ProfileEditDialog", "Change Icon"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("ProfileEditDialog", "Avatar"))
        self.cancelButton.setText(_translate("ProfileEditDialog", "Cancel"))
        self.updateButton.setText(_translate("ProfileEditDialog", "Update"))

